@@ -173,15 +173,17 @@ HEADER
 
             state $cache = {};
             $cache->{$ref} //= do {
-                $self->{header} .= "defined(&$target\::new) || require $target;\n";
+                $self->top_add("use $target;\n");
                 1;
             };
 
             return $target;
         }
 
-            $ref eq 'Sidef::Variable::ClassInit'     ? $self->_dump_class_name($obj)
-          : $ref eq 'Sidef::Variable::Struct'        ? $self->_dump_class_name($obj)
+        ($ref eq 'Sidef::Variable::Struct' || $ref eq 'Sidef::Variable::Subset')
+          ? ($self->_dump_class_name($obj) . refaddr($obj))
+          : $ref eq 'Sidef::Variable::ClassInit'
+          ? (ref($obj->{name}) ? $self->_dump_class_name($obj->{name}) : $self->_dump_class_name($obj))
           : $ref eq 'Sidef::Variable::Ref'           ? 'REF'
           : $ref eq 'Sidef::Types::Block::BlockInit' ? 'Sidef::Types::Block::Block'
           :                                            $ref;
@@ -221,7 +223,7 @@ HEADER
     sub _dump_var {
         my ($self, $var, $refaddr) = @_;
 
-        $var->{name} // return 'undef';
+        $var->{name} eq '' and return 'undef';
 
         # $var->{in_use} || exists($var->{value}) || exists($var->{ref_type}) || return 'undef';
 
@@ -321,15 +323,18 @@ HEADER
     sub _dump_class_attributes {
         my ($self, @attrs) = @_;
 
+        my %seen;
         my @code;
-        foreach my $attr (@attrs) {
+        foreach my $attr (reverse @attrs) {
 
             my @vars = @{$attr->{vars}};
             @vars || next;
 
-            my @dumped_vars = map { ref($_) ? $self->_dump_var($_) : $_ } @vars;
+            my @dumped_vars = map { ref($_) ? $self->_dump_var($_) : $_ } grep { !$seen{$_->{name}}++ } @vars;
 
-            push @code,
+            @dumped_vars || next;
+
+            unshift @code,
               (   'my('
                 . join(', ', @dumped_vars) . ')'
                 . (exists($attr->{args}) ? '=' . $self->deparse_args($attr->{args}) : ''));
@@ -402,7 +407,7 @@ HEADER
 
     sub _dump_array {
         my ($self, $ref, $array) = @_;
-        $ref . '->new(' . join(', ', map { $self->deparse_expr(ref($_) eq 'HASH' ? $_ : {self => $_}) } @{$array}) . ')';
+        $ref . '->new([' . join(', ', map { $self->deparse_expr(ref($_) eq 'HASH' ? $_ : {self => $_}) } @{$array}) . '])';
     }
 
     sub _dump_indices {
@@ -476,7 +481,22 @@ HEADER
                   . $self->_dump_string($_->{name})
                   . (exists($_->{slurpy})    ? (", slurpy => " . $_->{slurpy})                       : '')
                   . (exists($_->{ref_type})  ? (", type => " . $self->_dump_reftype($_->{ref_type})) : '')
+                  . (exists($_->{subset})    ? (", subset => " . $self->_dump_reftype($_->{subset})) : '')
                   . (exists($_->{has_value}) ? (', has_value => 1')                                  : '')
+                  . (
+                    exists($_->{subset_blocks}) ? (
+                       ', subset_blocks => [' . join(
+                           ', ',
+                           map {
+                                   'sub{'
+                                 . $self->_dump_sub_init_vars($_->{init_vars}{vars}[0])
+                                 . $self->deparse_generic('', ';', '', $_->{code}) . '}'
+                             } @{$_->{subset_blocks}}
+                         )
+                         . ']'
+                      )
+                    : ''
+                    )
                   . (
                      exists($_->{where_block})
                      ? (', where_block => sub{'
@@ -496,9 +516,26 @@ HEADER
           . join(', ', map { $self->_dump_string($vars[$_]{name}) . ' => ' . $_ } 0 .. $#vars) . '}';
     }
 
+    sub _get_inherited_stuff {
+        my ($self, $classes, $callback) = @_;
+
+        my @vars;
+        foreach my $class (@{$classes}) {
+            push @vars, $callback->($class);
+            if (exists $class->{inherit}) {
+                unshift @vars, $self->_get_inherited_stuff($class->{inherit}, $callback);
+            }
+        }
+
+        @vars;
+    }
+
     sub _dump_class_name {
         my ($self, $class) = @_;
-        join('::', $self->{environment_name}, $class->{class}, $class->{name});
+        join('::',
+             $self->{environment_name},
+             $class->{class} || 'main',
+             $class->{name}  || (Sidef::normalize_type(ref($class)) . refaddr($class)));
     }
 
     sub deparse_generic {
@@ -729,8 +766,9 @@ HEADER
             $code = $self->_dump_init_vars($obj);
         }
         elsif ($ref eq 'Sidef::Variable::ClassInit') {
+            my $name = $self->_get_reftype($obj);
             if ($addr{$refaddr}++) {
-                $code = q{'} . $self->_dump_class_name($obj) . q{'};
+                $code = q{'} . $name . q{'};
             }
             else {
                 my $block = $obj->{block};
@@ -739,20 +777,10 @@ HEADER
 
                 my $package_name;
                 if (ref $obj->{name}) {
-
-                    if (ref $obj->{name} eq 'HASH') {
-                        die "[ERROR] Invalid class name: '$obj->{name}' inside namespace '$obj->{class}'";
-                    }
-
                     $code .= ($package_name = $self->_get_reftype($obj->{name}));
                 }
                 else {
-
-                    if ($obj->{name} eq '') {
-                        $obj->{name} = '__FUNC__' . $refaddr;
-                    }
-
-                    $code .= ($package_name = $self->_dump_class_name($obj));
+                    $code .= ($package_name = $name);
                 }
 
                 local $self->{class}            = refaddr($block);
@@ -761,11 +789,10 @@ HEADER
                 local $self->{package_name}     = $package_name;
                 local $self->{inherit}          = $obj->{inherit} if exists $obj->{inherit};
                 local $self->{class_vars}       = $obj->{vars} if exists $obj->{vars};
-                local $self->{class_struct}     = $obj->{struct} if exists $obj->{struct};
                 local $self->{class_attributes} = $obj->{attributes} if exists $obj->{attributes};
                 local $self->{ref_class}        = 1 if ref($obj->{name});
                 $code .= $self->deparse_expr({self => $block});
-                $code .= '; ' . $self->_dump_string($package_name) . '}';
+                $code .= "; '$package_name'}";
             }
         }
         elsif ($ref eq 'Sidef::Types::Block::BlockInit') {
@@ -793,45 +820,82 @@ HEADER
 
                         if ($is_class) {
                             local $" = " ";
+                            my $class_name = $self->{class_name};
                             $code .= " " x $Sidef::SPACES;
-                            $code .= "use base qw("
-                              . (
-                                 exists($self->{inherit})
-                                 ? (join(' ', map { ref($_) ? $self->_dump_class_name($_) : $_ } @{$self->{inherit}}) . ' ')
-                                 : ''
-                                )
-                              . ($self->{package_name} eq 'Sidef::Object::Object' ? '' : "Sidef::Object::Object") . ");\n";
+
+                            my $base_pkgs = (
+                                             exists($self->{inherit})
+                                             ? (
+                                                join(' ',
+                                                     grep { $_ ne $class_name }
+                                                     map { ref($_) ? $self->_get_reftype($_) : $_ } @{$self->{inherit}})
+                                               )
+                                             : ''
+                                            )
+                              . (exists($self->{ref_class}) ? '' : ' Sidef::Object::Object');
+
+                            if ($base_pkgs ne '') {
+                                $code .= "use parent qw(-norequire $base_pkgs);\n";
+                            }
                         }
 
-                        if ($is_class and not $self->{ref_class}) {
+                        ## TODO: find a simpler and more elegant solution
+                        if ($is_class and not exists($self->{ref_class})) {
+
+                            my @class_vars = do {
+                                my %seen;
+                                reverse grep { !$seen{$_->{name}}++ }
+                                  reverse(
+                                          exists($self->{inherit})
+                                          ? $self->_get_inherited_stuff($self->{inherit},
+                                                                        sub { exists($_[0]->{vars}) ? @{$_[0]->{vars}} : () })
+                                          : (),
+                                          @{$self->{class_vars}}
+                                         );
+                            };
+
+                            my @class_attributes = do {
+                                my %seen;
+                                (
+                                 exists($self->{inherit})
+                                 ? $self->_get_inherited_stuff(
+                                                              $self->{inherit},
+                                                              sub { exists($_[0]->{attributes}) ? @{$_[0]->{attributes}} : () }
+                                   )
+                                 : (),
+                                 (exists($self->{class_attributes}) ? @{$self->{class_attributes}} : ())
+                                );
+                            };
 
                             $code .=
                               (" " x $Sidef::SPACES) . "\$new$refaddr = Sidef::Types::Block::Block->new(code => sub {" . "\n";
                             push @{$self->{function_declarations}}, [$refaddr, "my \$new$refaddr;"];
 
                             $Sidef::SPACES += $Sidef::SPACES_INCR;
-                            $code .= $self->_dump_sub_init_vars(@{$self->{class_vars}},
-                                                                (map { @{$_->{vars}} } @{$self->{class_struct}}))
-                              . $self->_dump_class_attributes(@{$self->{class_attributes}});
+                            $code .=
+                              $self->_dump_sub_init_vars(@class_vars) . $self->_dump_class_attributes(@class_attributes);
+
+                            my @class_var_attributes = do {
+                                my %seen;
+                                reverse(grep { !$seen{$_->{name}}++ } map { @{$_->{vars}} } reverse(@class_attributes));
+                            };
 
                             $code .= (' ' x $Sidef::SPACES) . 'my $self = bless {';
-                            foreach my $var (@{$self->{class_vars}},
-                                             (map { @{$_->{vars}} } @{$self->{class_attributes}}),
-                                             (map { @{$_->{vars}} } @{$self->{class_struct}})) {
+                            foreach my $var (@class_vars, @class_var_attributes) {
                                 $code .= qq{"\Q$var->{name}\E"=>} . $self->_dump_var($var) . ', ';
                             }
 
                             $code .=
                                 '}, __PACKAGE__;' . "\n"
                               . (' ' x $Sidef::SPACES)
-                              . '$self->init() if $self->can("init");' . "\n"
+                              . 'if(defined(my $sub = $self->can("init"))) { $sub->($self) }' . "\n"
                               . (' ' x $Sidef::SPACES)
                               . '$self;' . "\n";
 
                             $Sidef::SPACES -= $Sidef::SPACES_INCR;
                             $code .=
                                 (" " x $Sidef::SPACES . "}") . ', '
-                              . $self->_dump_var_attr(@{$self->{class_vars}}, map { @{$_->{vars}} } @{$self->{class_struct}})
+                              . $self->_dump_var_attr(@class_vars)
                               . ", type => "
                               . $self->_dump_string('class')
                               . ", name => "
@@ -843,9 +907,7 @@ HEADER
                               . $self->_dump_string("$self->{package_name}\::call")
                               . "} = sub { CORE::shift(\@_); \$new$refaddr->call(\@_) } };\n";
 
-                            foreach my $var (@{$self->{class_vars}},
-                                             (map { @{$_->{vars}} } @{$self->{class_attributes}}),
-                                             (map { @{$_->{vars}} } @{$self->{class_struct}})) {
+                            foreach my $var (@class_vars, @class_var_attributes) {
                                 $code .=
                                   (' ' x $Sidef::SPACES) . qq{sub $var->{name} : lvalue { \$_[0]->{"\Q$var->{name}\E"} }\n};
                             }
@@ -939,14 +1001,14 @@ HEADER
             ## ok
         }
         elsif ($ref eq 'Sidef::Variable::Struct') {
-            my $name = $self->_dump_class_name($obj);
+            my $name = $self->_get_reftype($obj);
             if ($addr{$refaddr}++) {
-                $code = $name;
+                $code = "'$name'";
             }
             else {
                 $Sidef::SPACES += $Sidef::SPACES_INCR;
                 $code =
-                    "package $name {\n"
+                    "do { package $name {\n"
                   . (' ' x $Sidef::SPACES)
                   . "\$new$refaddr = Sidef::Types::Block::Block->new(code => sub {" . "\n"
                   . (' ' x $Sidef::SPACES)
@@ -971,15 +1033,44 @@ HEADER
                   . join("\n" . (' ' x $Sidef::SPACES),
                          map { "sub $_->{name} : lvalue { \$_[0]->{$_->{name}} }" } @{$obj->{vars}})
                   . "\n"
-                  . (' ' x ($Sidef::SPACES - $Sidef::SPACES_INCR)) . "}";
+                  . (' ' x ($Sidef::SPACES - $Sidef::SPACES_INCR))
+                  . "}; '$name'}";
 
                 push @{$self->{function_declarations}}, [$refaddr, "my \$new$refaddr;"];
 
                 $Sidef::SPACES -= $Sidef::SPACES_INCR;
             }
         }
+        elsif ($ref eq 'Sidef::Variable::Subset') {
+            my $name = $self->_get_reftype($obj);
+            if ($addr{$refaddr}++) {
+                $code = "'$name'";
+            }
+            else {
+                my @parents = map { $self->_get_reftype($_) } @{$obj->{inherits}};
+                $code = qq{
+                    do {
+                        package $name {
+                            use parent qw(-norequire @parents)
+                        };
+                        '$name'
+                    }
+                };
+            }
+        }
         elsif ($ref eq 'Sidef::Types::Number::Number') {
-            $code = $self->make_constant($ref, 'new', "Number$refaddr", "'" . $obj->_get_frac . "'");
+            if ($obj->is_zero) {
+                $code = '(Sidef::Types::Number::Number::ZERO)';
+            }
+            elsif ($obj->is_one) {
+                $code = '(Sidef::Types::Number::Number::ONE)';
+            }
+            elsif ($obj->is_mone) {
+                $code = '(Sidef::Types::Number::Number::MONE)';
+            }
+            else {
+                $code = $self->make_constant($ref, 'new', "Number$refaddr", "'" . $obj->_get_frac . "'");
+            }
         }
         elsif ($ref eq 'Sidef::Types::Number::Inf') {
             $code = $self->make_constant($ref, 'new', "Inf$refaddr");
@@ -1032,73 +1123,35 @@ HEADER
 
             my $expr = $self->deparse_expr({self => $obj->{expr}});
 
+            local $obj->{block}{init_vars} = bless({vars => $obj->{vars}}, 'Sidef::Variable::Init');
+            my $block = $self->deparse_expr({self => $obj->{block}});
+
             if (
                 @{$obj->{vars}} > 1
                 or (@{$obj->{vars}} == 1
                     and exists($obj->{vars}[0]{slurpy}))
               ) {
 
-                my $last_var = $obj->{vars}[-1];
-
-                my $hash  = 0;
-                my $array = 0;
-                if (exists($last_var->{slurpy})) {
-                    delete($last_var->{slurpy});
-                    if (exists($last_var->{array})) {
-                        $array = delete($last_var->{array});
-                    }
-                    else {
-                        $hash = delete($last_var->{hash});
-                    }
-                }
-
-                my @vars = map { $self->deparse_expr({self => $_}) } @{$obj->{vars}};
-
-                my @last_var = ($array ? ($vars[-1] =~ tr/$/@/r) : $hash ? ($vars[-1] =~ tr/$/%/r) : ());
-                my $init_vars = '(' . join(',', @vars, @last_var) . ')';
-                my $vars = '(' . join(',', @vars[0 .. $#vars - @last_var], @last_var) . ')';
-                my $block = $self->deparse_bare_block($obj->{block}->{code});
-
                 $code =
                     'do { my $obj = '
-                  . $expr . '; my '
-                  . $init_vars . '; '
-                  . 'my $block = CORE::sub '
+                  . $expr . ';'
+                  . 'my $block = '
                   . $block . ';'
                   . 'for my $group(ref($obj) ? $obj->SUPER::isa("ARRAY") ? @$obj : @{$obj->to_a} : ()) {'
-                  . $vars
-                  . ' = (ref($group) && $group->SUPER::isa("ARRAY") ? @$group'
-                  . ' : $group);'
-                  . (
-                     @last_var
-                     ? ($vars[-1] . ' = '
-                        . ($array ? 'Sidef::Types::Array::Array' : 'Sidef::Types::Hash::Hash')
-                        . '->new('
-                        . join('', @last_var) . ')')
-                     : ''
-                    )
-                  . '; $block->() }}';
+                  . '; $block->call((ref($group) && $group->SUPER::isa("ARRAY") ? @$group : $group)) }}';
             }
             else {
 
-                my @vars = map { $self->deparse_expr({self => $_}) } @{$obj->{vars}};
-                my $block = $self->deparse_bare_block($obj->{block}->{code});
-
-                my $var = $vars[0];
-
                 $code =
                     'do { my $obj = '
-                  . $expr . '; my '
-                  . $var
-                  . '; my $block = CORE::sub '
+                  . $expr . ';'
+                  . '; my $block = '
                   . $block . ';'
-                  . 'if (ref($obj)) { if (defined(my $sub = $obj->SUPER::can("range_iter"))) { my $iter = $sub->($obj);'
-                  . 'while (defined('
-                  . $var
-                  . '  = $iter->())) { $block->()} }'
+                  . 'if (ref($obj)) { if (defined(my $sub = $obj->SUPER::can("iter"))) { my $iter = $sub->($obj);'
+                  . 'while (defined(my $item'
+                  . ' = $iter->run )) { $block->call($item) } }'
                   . 'else { for my $item ($obj->SUPER::isa("ARRAY") ? @$obj : @{$obj->to_a}) { '
-                  . $var
-                  . ' = $item; $block->() }}}}';
+                  . '$block->call($item) }}}}';
             }
         }
         elsif ($ref eq 'Sidef::Types::Bool::Ternary') {
@@ -1162,10 +1215,11 @@ HEADER
               . ' else { } }';
         }
         elsif ($ref eq 'Sidef::Types::Block::Gather') {
+            $self->top_add("use Sidef::Types::Array::Array;\n");
             $code =
                 "do {my \@_$refaddr;"
               . $self->deparse_bare_block($obj->{block}->{code})
-              . "; Sidef::Types::Array::Array->new(\@_$refaddr)}";
+              . "; bless(\\\@_$refaddr, 'Sidef::Types::Array::Array')}";
         }
         elsif ($ref eq 'Sidef::Types::Block::Take') {
             my $raddr = refaddr($obj->{gather});
@@ -1193,33 +1247,30 @@ HEADER
                     $code = "goto END$self->{function}";
                 }
                 else {
-                    $code = 'return Sidef::Types::Block::Return->new';
+                    $code = "return;";
                 }
             }
         }
         elsif ($ref eq 'Sidef::Math::Math') {
             $code = $self->make_constant($ref, 'new', "Math$refaddr");
         }
-        elsif ($ref eq 'Sidef::Types::Glob::FileHandle') {
-            if ($obj->{fh} eq \*STDIN) {
-                $code = $self->make_constant($ref, 'new', "STDIN$refaddr", 'fh => \*STDIN');
-            }
-            elsif ($obj->{fh} eq \*STDOUT) {
-                $code = $self->make_constant($ref, 'new', "STDOUT$refaddr", 'fh => \*STDOUT');
-            }
-            elsif ($obj->{fh} eq \*STDERR) {
-                $code = $self->make_constant($ref, 'new', "STDERR$refaddr", 'fh => \*STDERR');
-            }
-            elsif ($obj->{fh} eq \*ARGV) {
-                $code = $self->make_constant($ref, 'new', "ARGF$refaddr", 'fh => \*ARGV');
-            }
-            else {
-                my $data = $self->_dump_string(
-                                               do { seek($obj->{fh}, 0, 0); local $/; readline($obj->{fh}) }
-                                              );
-                $code =
-                  $self->make_constant($ref, 'new', "DATA$refaddr", qq{fh => do {open my \$fh, '<:utf8', \\$data; \$fh}});
-            }
+        elsif ($ref eq 'Sidef::Meta::Glob::STDIN') {
+            $code = $self->make_constant('Sidef::Types::Glob::FileHandle', 'new', "STDIN$refaddr", 'fh => \*STDIN');
+        }
+        elsif ($ref eq 'Sidef::Meta::Glob::STDOUT') {
+            $code = $self->make_constant('Sidef::Types::Glob::FileHandle', 'new', "STDOUT$refaddr", 'fh => \*STDOUT');
+        }
+        elsif ($ref eq 'Sidef::Meta::Glob::STDERR') {
+            $code = $self->make_constant('Sidef::Types::Glob::FileHandle', 'new', "STDERR$refaddr", 'fh => \*STDERR');
+        }
+        elsif ($ref eq 'Sidef::Meta::Glob::ARGF') {
+            $code = $self->make_constant('Sidef::Types::Glob::FileHandle', 'new', "ARGF$refaddr", 'fh => \*ARGV');
+        }
+        elsif ($ref eq 'Sidef::Meta::Glob::DATA') {
+            require Encode;
+            my $data = $self->_dump_string(Encode::encode_utf8(${$obj->{data}}));
+            $code = $self->make_constant('Sidef::Types::Glob::FileHandle',
+                                         'new', "DATA$refaddr", qq{fh => do {open my \$fh, '<:utf8', \\$data; \$fh}});
         }
         elsif ($ref eq 'Sidef::Variable::Magic') {
             $code = $obj->{name};
@@ -1337,18 +1388,12 @@ HEADER
             $code = qq~((CORE::warn(@args, " at \Q$obj->{file}\E line $obj->{line}\\n")) ? ~
               . qq~(Sidef::Types::Bool::Bool::FALSE) : (Sidef::Types::Bool::Bool::TRUE))~;
         }
-        elsif ($ref eq 'Sidef::Types::Array::MultiArray') {
-            $code = $self->make_constant($ref, 'new', "MultiArr$refaddr");
-        }
         elsif ($ref eq 'Sidef::Types::Glob::Pipe') {
             $code = $self->make_constant($ref, 'new', "Pipe$refaddr", map { $self->_dump_string($_) } @{$obj});
         }
         elsif ($ref eq 'Sidef::Parser') {
             $code = $self->make_constant($ref, 'new', "Parser$refaddr");
         }
-        elsif ($ref eq 'Sidef') {
-            $code = $self->make_constant($ref, 'new', "Sidef$refaddr");
-        }
         elsif ($ref eq 'Sidef::Perl::Perl') {
             $code = $self->make_constant($ref, 'new', "Perl$refaddr");
         }
@@ -1356,6 +1401,7 @@ HEADER
             $code = qq{CORE::die "Unimplemented at " . } . $self->_dump_string($obj->{file}) . qq{. " line $obj->{line}\\n"};
         }
         elsif (exists $self->{data_types}{$ref}) {
+            $self->top_add("use $self->{data_types}{$ref};\n");
             $code = "'" . $self->{data_types}{$ref} . "'";
         }
 
@@ -1422,7 +1468,7 @@ HEADER
                         $code .= 'goto ' . "END$self->{function}}";
                     }
                     else {
-                        $code .= 'return Sidef::Types::Block::Return->new' . $self->deparse_args(@{$call->{arg}});
+                        $code .= 'return' . $self->deparse_args(@{$call->{arg}});
                     }
 
                     next;
