@@ -41,6 +41,28 @@ package Sidef::Deparse::Sidef {
                   Sidef::DataTypes::Object::Object        Object
                   Sidef::DataTypes::Sidef::Sidef          Sidef
                   Sidef::DataTypes::Variable::LazyMethod  LazyMethod
+
+                  Sidef::Math::Math                       Math
+                  Sidef::Meta::Glob::ARGF                 ARGF
+                  Sidef::Meta::Glob::STDIN                STDIN
+                  Sidef::Meta::Glob::STDOUT               STDOUT
+                  Sidef::Meta::Glob::STDERR               STDERR
+                  Sidef::Types::Number::Inf               Inf
+                  Sidef::Types::Number::Ninf              Inf.neg
+                  Sidef::Types::Number::Nan               NaN
+                  Sidef::Parser                           Parser
+
+                  Sidef::Types::Nil::Nil                  nil
+                  Sidef::Types::Null::Null                null
+                  Sidef::Types::Block::Next               next
+                  Sidef::Types::Block::Break              break
+                  Sidef::Types::Block::Continue           continue
+
+                  Sidef::Perl::Perl                       Perl
+                  Sidef::Time::Time                       Time
+                  Sidef::Sys::Sig                         Sig
+                  Sidef::Meta::Unimplemented              ...
+
                   )
             },
             %args,
@@ -96,8 +118,12 @@ package Sidef::Deparse::Sidef {
 
         my $ref = ref($obj);
 
-            $ref eq 'Sidef::Variable::ClassInit'     ? $obj->{name}
-          : $ref eq 'Sidef::Variable::Struct'        ? $obj->{name}
+        if (exists $self->{data_types}{$ref}) {
+            return $self->{data_types}{$ref};
+        }
+
+        ($ref eq 'Sidef::Variable::ClassInit' || $ref eq 'Sidef::Variable::Struct' || $ref eq 'Sidef::Variable::Subset')
+          ? $obj->{name}
           : $ref eq 'Sidef::Types::Block::BlockInit' ? 'Block'
           :                                            substr($ref, rindex($ref, '::') + 2);
     }
@@ -111,6 +137,7 @@ package Sidef::Deparse::Sidef {
                   . (exists($_->{class}) && $_->{class} ne $self->{class} ? $_->{class} . '::' : '')
                   . (exists($_->{ref_type}) ? ($self->_dump_reftype($_->{ref_type}) . ' ') : '')
                   . $_->{name}
+                  . (exists($_->{subset}) ? (' < ' . $self->_dump_reftype($_->{subset})) : '')
                   . (exists($_->{where_block}) ? $self->deparse_expr({self => $_->{where_block}}) : '')
                   . (exists($_->{where_expr}) ? ('(' . $self->deparse_expr({self => $_->{where_expr}}) . ')') : '')
                   . (exists($_->{value}) ? ('=(' . $self->deparse_expr({self => $_->{value}}) . ')') : '')
@@ -170,7 +197,7 @@ package Sidef::Deparse::Sidef {
                 $code =
                   $obj->{name} =~ /^[0-9]+\z/
                   ? ('$' . $obj->{name})
-                  : (($obj->{class} ne $self->{class} ? $obj->{class} . '::' : '') . $obj->{name});
+                  : (($obj->{class} ne $self->{class} ? ($obj->{class} . '::') : '') . $obj->{name});
             }
             elsif ($obj->{type} eq 'func' or $obj->{type} eq 'method') {
                 if ($addr{refaddr($obj)}++) {
@@ -226,6 +253,17 @@ package Sidef::Deparse::Sidef {
                 $code = "struct $obj->{name} {" . $self->_dump_vars(@{$obj->{vars}}) . '}';
             }
         }
+        elsif ($ref eq 'Sidef::Variable::Subset') {
+            if ($addr{refaddr($obj)}++) {
+                $code = $obj->{name};
+            }
+            else {
+                $code =
+                    "subset $obj->{name} < "
+                  . join(',', map { $self->deparse_expr({self => $_}) } @{$obj->{inherits}})
+                  . (exists($obj->{blocks}) ? (' ' . $self->deparse_expr({self => $obj->{blocks}[-1]})) : '');
+            }
+        }
         elsif ($ref eq 'Sidef::Variable::Local') {
             $code = 'local ' . '(' . $self->deparse_script($obj->{expr}) . ')';
         }
@@ -251,7 +289,7 @@ package Sidef::Deparse::Sidef {
                 }
             }
             else {
-                $code = $name;
+                $code = ($obj->{class} ne $self->{class} ? ($obj->{class} . '::') : '') . $name;
             }
         }
         elsif ($ref eq 'Sidef::Variable::Const') {
@@ -264,30 +302,31 @@ package Sidef::Deparse::Sidef {
                 }
             }
             else {
-                $code = $name;
+                $code = ($obj->{class} ne $self->{class} ? ($obj->{class} . '::') : '') . $name;
             }
         }
         elsif ($ref eq 'Sidef::Variable::Static') {
             my $name = $obj->{name};
             if (not exists $obj->{inited}) {
                 $obj->{inited} = 1;
-                $code = "static $name";
+                $code = "static " . ($obj->{class} ne $self->{class} ? ($obj->{class} . '::') : '') . $name;
                 if (defined $obj->{expr}) {
                     $code .= ' = (' . $self->deparse_script($obj->{expr}) . ')';
                 }
             }
             else {
-                $code = $name;
+                $code = ($obj->{class} ne $self->{class} ? ($obj->{class} . '::') : '') . $name;
             }
         }
         elsif ($ref eq 'Sidef::Variable::ClassInit') {
             if ($addr{refaddr($obj)}++) {
-                $code =
-                  $self->_dump_class_name(
-                                     $obj->{name} eq ''
-                                     ? '__CLASS__'
-                                     : ($obj->{class} ne $self->{class} ? ($obj->{class} . '::' . $obj->{name}) : $obj->{name})
-                  );
+                $code = (
+                         $obj->{name} eq '' ? '__CLASS__'
+                         : $self->_dump_class_name(
+                                                     $obj->{class} ne $self->{class} ? ($obj->{class} . '::' . $obj->{name})
+                                                   : $obj->{name}
+                                                  )
+                        );
             }
             else {
                 my $block     = $obj->{block};
@@ -300,13 +339,14 @@ package Sidef::Deparse::Sidef {
                 }
 
                 local $self->{class} = $obj->{class};
-                $code .= "class " . $self->_dump_class_name($obj->{name});
+                my $name = $self->_dump_class_name($obj->{name});
+                $code .= "class " . $name;
                 $code .= '(' . $self->_dump_vars(@{$obj->{vars}}) . ')';
                 if (exists $obj->{inherit}) {
-                    $code .= ' << ' . join(', ', map { $self->deparse_expr({self => $_}) } @{$obj->{inherit}}) . ' ';
-                }
-                elsif (exists $obj->{struct}) {
-                    $code .= ' << ' . join(', ', map { $self->deparse_expr({self => $_}) } @{$obj->{struct}}) . ' ';
+                    my $inherited = join(', ', grep { $_ ne $name } map { $self->_dump_class_name($_) } @{$obj->{inherit}});
+                    if ($inherited ne '') {
+                        $code .= ' < ' . $inherited . ' ';
+                    }
                 }
                 $code .= $self->deparse_expr({self => $block});
 
@@ -374,27 +414,15 @@ package Sidef::Deparse::Sidef {
         elsif ($ref eq 'Sidef::Meta::Warning') {
             $code = 'warn' . $self->deparse_args($obj->{arg});
         }
-        elsif ($ref eq 'Sidef::Meta::Unimplemented') {
-            $code = '...';
-        }
         elsif ($ref eq 'Sidef::Eval::Eval') {
             $code = 'eval' . $self->deparse_args($obj->{expr});
         }
-        elsif ($ref eq 'Sidef::Parser') {
-            $code = 'Parser';
-        }
         elsif ($ref eq 'Sidef::Variable::NamedParam') {
             $code = $obj->[0] . ':' . $self->deparse_args(@{$obj->[1]});
         }
         elsif ($ref eq 'Sidef::Variable::Label') {
             $code = '@:' . $obj->{name};
         }
-        elsif ($ref eq 'Sidef::Variable::LazyMethod') {
-            $code = 'LazyMethod';
-        }
-        elsif ($ref eq 'Sidef::Types::Block::Break') {
-            $code = 'break';
-        }
         elsif ($ref eq 'Sidef::Types::Block::Given') {
             $code = 'given ' . $self->deparse_args($obj->{expr}) . $self->deparse_expr({self => $obj->{block}});
         }
@@ -410,12 +438,6 @@ package Sidef::Deparse::Sidef {
         elsif ($ref eq 'Sidef::Types::Block::With') {
             $code = 'with ' . $self->deparse_args($obj->{expr}) . $self->deparse_expr({self => $obj->{block}});
         }
-        elsif ($ref eq 'Sidef::Types::Block::Next') {
-            $code = 'next';
-        }
-        elsif ($ref eq 'Sidef::Types::Block::Continue') {
-            $code = 'continue';
-        }
         elsif ($ref eq 'Sidef::Types::Block::Return') {
             if (not exists $expr->{call}) {
                 $code = 'return';
@@ -479,35 +501,11 @@ package Sidef::Deparse::Sidef {
               . $self->deparse_expr({self => $obj->{expr}}) . ') '
               . $self->deparse_bare_block($obj->{block}->{code});
         }
-        elsif ($ref eq 'Sidef::Math::Math') {
-            $code = 'Math';
-        }
-        elsif ($ref eq 'Sidef::Types::Glob::FileHandle') {
-            if ($obj->{fh} eq \*STDIN) {
-                $code = 'STDIN';
-            }
-            elsif ($obj->{fh} eq \*STDOUT) {
-                $code = 'STDOUT';
-            }
-            elsif ($obj->{fh} eq \*STDERR) {
-                $code = 'STDERR';
-            }
-            elsif ($obj->{fh} eq \*ARGV) {
-                $code = 'ARGF';
-            }
-            else {
-                $code = 'DATA';
-                if (not exists $addr{$obj->{fh}}) {
-                    my $orig_pos = tell($obj->{fh});
-                    seek($obj->{fh}, 0, 0);
-                    $self->{after} .= "\n__DATA__\n" . do {
-                        local $/;
-                        require Encode;
-                        Encode::decode_utf8(readline($obj->{fh}));
-                    };
-                    seek($obj->{fh}, $orig_pos, 0);
-                    $addr{$obj->{fh}} = 1;
-                }
+        elsif ($ref eq 'Sidef::Meta::Glob::DATA') {
+            $code = 'DATA';
+            if (not exists $addr{$obj->{data}}) {
+                $self->{after} .= "\n__DATA__\n" . ${$obj->{data}};
+                $addr{$obj->{data}} = 1;
             }
         }
         elsif ($ref eq 'Sidef::Variable::Magic') {
@@ -516,36 +514,15 @@ package Sidef::Deparse::Sidef {
         elsif ($ref eq 'Sidef::Types::Hash::Hash') {
             $code = keys(%{$obj}) ? $obj->dump->get_value : 'Hash';
         }
-        elsif ($ref eq 'Sidef::Perl::Perl') {
-            $code = 'Perl';
-        }
-        elsif ($ref eq 'Sidef::Time::Time') {
-            $code = 'Time';
-        }
-        elsif ($ref eq 'Sidef::Sys::Sig') {
-            $code = 'Sig';
-        }
         elsif ($ref eq 'Sidef::Types::Number::Number') {
             $code = $self->_dump_number($obj->_get_frac);
             if (index($code, '/') != -1) {
                 $code = qq{Number("$code")};
             }
         }
-        elsif ($ref eq 'Sidef::Types::Number::Inf') {
-            $code = 'Inf';
-        }
-        elsif ($ref eq 'Sidef::Types::Number::Ninf') {
-            $code = 'Inf.neg';
-        }
-        elsif ($ref eq 'Sidef::Types::Number::Nan') {
-            $code = 'NaN';
-        }
         elsif ($ref eq 'Sidef::Types::Array::Array' or $ref eq 'Sidef::Types::Array::HCArray') {
             $code = $self->_dump_array($obj);
         }
-        elsif ($ref eq 'Sidef::Types::Nil::Nil') {
-            $code = 'nil';
-        }
         elsif (exists $self->{data_types}{$ref}) {
             $code = $self->{data_types}{$ref};
         }
