@@ -39,14 +39,15 @@ package Sidef::Object::Object {
             ($obj1, $obj2) = ($obj2, $obj1);
         }
 
-        if (CORE::ref($obj1) && $obj1->SUPER::isa(CORE::ref($obj2)) or CORE::ref($obj2) && $obj2->SUPER::isa(CORE::ref($obj1)))
-        {
+        if (   CORE::ref($obj1) && $obj1->SUPER::isa(CORE::ref($obj2))
+            or CORE::ref($obj2) && $obj2->SUPER::isa(CORE::ref($obj1))) {
             if (defined(my $sub = $obj1->can('<=>'))) {
                 return $sub->($obj1, $obj2);
             }
         }
 
-        my $cmp = Scalar::Util::refaddr($obj1) <=> (CORE::ref($obj2) ? Scalar::Util::refaddr($obj2) : 'inf');
+        my $cmp = ((ref($obj1) ? Scalar::Util::refaddr($obj1) : ('-inf' + 0))
+                   <=> (CORE::ref($obj2) ? Scalar::Util::refaddr($obj2) : ('-inf' + 0)));
 
             $cmp < 0 ? Sidef::Types::Number::Number::MONE
           : $cmp > 0 ? Sidef::Types::Number::Number::ONE
@@ -59,13 +60,14 @@ package Sidef::Object::Object {
          || $obj2->SUPER::isa(CORE::ref($obj1) || return (Sidef::Types::Bool::Bool::FALSE)))
           || return (Sidef::Types::Bool::Bool::FALSE);
 
+        return Sidef::Types::Bool::Bool::TRUE
+          if (Scalar::Util::refaddr($obj1) == Scalar::Util::refaddr($obj2));
+
         if (defined(my $sub = $obj1->can('=='))) {
             return $sub->($obj1, $obj2);
         }
 
-        Scalar::Util::refaddr($obj1) == Scalar::Util::refaddr($obj2)
-          ? (Sidef::Types::Bool::Bool::TRUE)
-          : (Sidef::Types::Bool::Bool::FALSE);
+        ($obj1 cmp $obj2)->is_zero;
       };
 
     sub new {
@@ -80,6 +82,11 @@ package Sidef::Object::Object {
 
     *println = \&say;
 
+    sub lazy {
+        my ($self) = @_;
+        Sidef::Object::Lazy->new(obj => $self);
+    }
+
     sub print {
         (CORE::print @_)
           ? (Sidef::Types::Bool::Bool::TRUE)
@@ -153,20 +160,11 @@ package Sidef::Object::Object {
             @parents;
         };
 
-        Sidef::Types::Array::Array->new(map { Sidef::Types::String::String->new($_) } $extract_parents->(CORE::ref($obj)));
+        Sidef::Types::Array::Array->new([map { Sidef::Types::String::String->new($_) } $extract_parents->(CORE::ref($obj))]);
     }
 
     sub interpolate {
-        my ($self, @args) = @_;
-        $self->new(
-            CORE::join(
-                '',
-                map {
-                    eval { ${CORE::ref($_) ne 'Sidef::Types::String::String' ? $_->to_s : $_} }
-                      // $_
-                  } @args
-            )
-        );
+        $_[0]->new(CORE::join('', @_[1 .. $#_]));
     }
 
     {
@@ -256,21 +254,11 @@ package Sidef::Object::Object {
                 or $f_type eq 'Sidef::Types::Glob::File'
                 or $f_type eq 'Sidef::Types::Glob::Dir') {
 
-                # String ~~ Array
-                if ($s_type eq 'Sidef::Types::Array::Array') {
-                    return $second->contains($first);
-                }
-
                 # String ~~ RangeString
                 if ($s_type eq 'Sidef::Types::Range::RangeString') {
                     return $second->contains($first);
                 }
 
-                # String ~~ Hash
-                if ($s_type eq 'Sidef::Types::Hash::Hash') {
-                    return $second->exists($first);
-                }
-
                 # String ~~ String
                 if ($s_type eq 'Sidef::Types::String::String') {
                     return $second->eq($first);
@@ -378,14 +366,17 @@ package Sidef::Object::Object {
             # Second is Array
             if ($s_type eq 'Sidef::Types::Array::Array') {
 
-                if ($f_type eq 'Sidef::Types::Array::Array') {
-                    return $first->eq($second);
-                }
-
                 # Any ~~ Array
                 return $second->contains($first);
             }
 
+            # Second is Hash
+            if ($s_type eq 'Sidef::Types::Hash::Hash') {
+
+                # Any ~~ Hash
+                return $second->exists($first);
+            }
+
             ($first eq $second)
               ? (Sidef::Types::Bool::Bool::TRUE)
               : (Sidef::Types::Bool::Bool::FALSE);
