@@ -4,47 +4,116 @@ package Sidef::Optimizer {
     use Scalar::Util qw(refaddr);
 
     use constant {
-                  STRING  => 'Sidef::Types::String::String',
-                  NUMBER  => 'Sidef::Types::Number::Number',
-                  REGEX   => 'Sidef::Types::Regex::Regex',
-                  BOOL    => 'Sidef::Types::Bool::Bool',
-                  ARRAY   => 'Sidef::Types::Array::Array',
-                  MATH    => 'Sidef::Math::Math',
-                  FILE    => 'Sidef::Types::Glob::File',
-                  SOCKET  => 'Sidef::Types::Glob::Socket',
-                  COMPLEX => 'Sidef::Types::Number::Complex',
+                  STRING      => 'Sidef::Types::String::String',
+                  NUMBER      => 'Sidef::Types::Number::Number',
+                  REGEX       => 'Sidef::Types::Regex::Regex',
+                  BOOL        => 'Sidef::Types::Bool::Bool',
+                  ARRAY       => 'Sidef::Types::Array::Array',
+                  COMPLEX     => 'Sidef::Types::Number::Complex',
+                  NUMBER_DT   => 'Sidef::DataTypes::Number::Number',
+                  STRING_DT   => 'Sidef::DataTypes::String::String',
+                  COMPLEX_DT  => 'Sidef::DataTypes::Number::Complex',
+                  REGEX_DT    => 'Sidef::DataTypes::Regex::Regex',
+                  RANGENUM_DT => 'Sidef::DataTypes::Range::RangeNumber',
+                  BACKTICK_DT => 'Sidef::DataTypes::Glob::Backtick',
                  };
 
+    my %cache;
     {
-        my %cache;
 
         sub methods {
             my ($package, @names) = @_;
-            my $module = $cache{$package} //= (($package =~ s{::}{/}gr) . '.pm');
+            my $module = ($cache{$package} //= (($package =~ s{::}{/}gr) . '.pm'));
             exists($INC{$module}) || require($module);
             map {
+                $cache{$package, $_} //= do {
+                    defined(my $method = $package->SUPER::can($_))
+                      or die "[ERROR] Invalid method $package: $_";
+                    $method;
+                  }
+            } @names;
+        }
 
-                defined(&{$package . '::' . $_})
-                  or die "Invalid method $package: $_";
+        my %table = (
+            qw(
+              Sidef::DataTypes::Bool::Bool            Sidef::Types::Bool::Bool
+              Sidef::DataTypes::Array::Array          Sidef::Types::Array::Array
+              Sidef::DataTypes::Array::Pair           Sidef::Types::Array::Pair
+              Sidef::DataTypes::Array::MultiArray     Sidef::Types::Array::MultiArray
+              Sidef::DataTypes::Hash::Hash            Sidef::Types::Hash::Hash
+              Sidef::DataTypes::Regex::Regex          Sidef::Types::Regex::Regex
+              Sidef::DataTypes::String::String        Sidef::Types::String::String
+              Sidef::DataTypes::Number::Number        Sidef::Types::Number::Number
+              Sidef::DataTypes::Number::Complex       Sidef::Types::Number::Complex
+              Sidef::DataTypes::Range::RangeNumber    Sidef::Types::Range::RangeNumber
+              Sidef::DataTypes::Range::RangeString    Sidef::Types::Range::RangeString
+              Sidef::DataTypes::Glob::Backtick        Sidef::Types::Glob::Backtick
+              Sidef::DataTypes::Glob::Socket          Sidef::Types::Glob::Socket
+              Sidef::DataTypes::Glob::Pipe            Sidef::Types::Glob::Pipe
+              Sidef::DataTypes::Glob::Dir             Sidef::Types::Glob::Dir
+              Sidef::DataTypes::Glob::File            Sidef::Types::Glob::File
+              Sidef::DataTypes::Variable::LazyMethod  Sidef::Variable::LazyMethod
+              )
+        );
 
-                \&{$package . '::' . $_}
+        my %seen;
+
+        sub dtypes {
+            my ($type, @names) = @_;
+
+            exists($table{$type}) || die "[ERROR] Non-existent data type: $type";
+
+            my $package = $table{$type};
+            my $module = ($cache{$package} //= (($package =~ s{::}{/}gr) . '.pm'));
+            exists($INC{$module}) || require($module);
+
+            if (not $seen{$type}++) {
+                no strict 'refs';
+                push @{$type . '::' . 'ISA'}, $package;
+            }
+
+            map {
+                $cache{$type, $_} //= do {
+                    defined(my $method = $type->SUPER::can($_))
+                      or die "[ERROR] Invalid method $type: $_";
+                    $method;
+                  }
             } @names;
         }
     }
 
     sub table {
-        scalar {map { $_ => 1 } @_};
+        [@_];
+    }
+
+    # It's probably easier to use a Cartesian product here,
+    # but for our purposes, it's good enough. At least for now.
+    sub build_tree {
+        my (@data) = @_;
+
+        my %tree;
+        foreach my $node (@data) {
+            my $ref = ($tree{$node->[0]} //= {});
+            $ref = $ref->{$#{$node->[1]}} //= {};
+            my $orig = $ref;
+            foreach my $arg (@{$node->[1]}) {
+                my $ref2 = $ref;
+                foreach my $key (@{$arg}) {
+                    $ref = $ref2->{$key} //= {};
+                }
+            }
+        }
+
+        \%tree;
     }
 
     my %rules = (
-        (STRING) => [
+        (STRING) => build_tree(
 
             # String.method(String)
             (
-             map {
-                 { $_, [table(STRING)] }
-               } methods(STRING, qw(
-                   new call
+             map { [$_, [table(STRING)]] }
+               methods(STRING, qw(
                    to downto
 
                    concat
@@ -78,9 +147,8 @@ package Sidef::Optimizer {
 
             # String.method()
             (
-             map {
-                 { $_, [] }
-               } methods(STRING, qw(
+             map { [$_, []] }
+               methods(STRING, qw(
                    lc uc fc tc wc tclc lcfirst
 
                    pop
@@ -111,6 +179,7 @@ package Sidef::Optimizer {
                    reverse
                    clear
                    sort
+                   split
 
                    is_empty
                    is_palindrome
@@ -136,12 +205,12 @@ package Sidef::Optimizer {
 
             # String.method(Number)
             (
-             map {
-                 { $_, [table(NUMBER)] }
-               } methods(STRING, qw(
+             map { [$_, [table(NUMBER)]] }
+               methods(STRING, qw(
                    eq ne
 
-                   times
+                   mul
+                   div
                    repeat
                    char
 
@@ -155,17 +224,12 @@ package Sidef::Optimizer {
             ),
 
             # String.method(String | Number | Regex)
-            (
-             map {
-                 { $_, [table(STRING, NUMBER, REGEX)] }
-               } methods(STRING, qw(split))
-            ),
+            (map { [$_, [table(STRING, NUMBER, REGEX)]] } methods(STRING, qw(split))),
 
             # String.method(String, String)
             (
-             map {
-                 { $_, [table(STRING), table(STRING)] }
-               } methods(STRING, qw(
+             map { [$_, [table(STRING), table(STRING)]] }
+               methods(STRING, qw(
                    tr
                    )
                )
@@ -173,9 +237,8 @@ package Sidef::Optimizer {
 
             # String.method(String, String, String)
             (
-             map {
-                 { $_, [table(STRING), table(STRING), table(STRING)] }
-               } methods(STRING, qw(
+             map { [$_, [table(STRING), table(STRING), table(STRING)]] }
+               methods(STRING, qw(
                    tr
                    )
                )
@@ -183,9 +246,8 @@ package Sidef::Optimizer {
 
             # String.method(String, Number)
             (
-             map {
-                 { $_, [table(STRING), table(NUMBER)] }
-               } methods(STRING, qw(
+             map { [$_, [table(STRING), table(NUMBER)]] }
+               methods(STRING, qw(
                    index
                    contains
                    sprintf
@@ -196,33 +258,21 @@ package Sidef::Optimizer {
 
             # String.method(String, Bool)
             (
-             map {
-                 { $_, [table(STRING), table(BOOL)] }
-               } methods(STRING, qw(
+             map { [$_, [table(STRING), table(BOOL)]] }
+               methods(STRING, qw(
                    range
                    )
                )
             ),
 
-        ],
+        ),
 
-        (NUMBER) => [
-
-            # Number.method(String | Number)
-            (
-             map {
-                 { $_, [table(STRING, NUMBER)] }
-               } methods(NUMBER, qw(
-                   new call
-                   )
-               )
-            ),
+        (NUMBER) => build_tree(
 
             # Number.method(Number)
             (
-             map {
-                 { $_, [table(NUMBER)] }
-               } methods(NUMBER, qw(
+             map { [$_, [table(NUMBER)]] }
+               methods(NUMBER, qw(
                    + - / * % %% **
 
                    lt gt le ge cmp acmp
@@ -253,22 +303,10 @@ package Sidef::Optimizer {
                )
             ),
 
-            # Number.method(String, Number)
-            (
-             map {
-                 { $_, [table(STRING), table(NUMBER)] }
-               } methods(NUMBER, qw(
-                   new call
-                   )
-               )
-            ),
-
             # Number.method()
             (
-             map {
-                 { $_, [] }
-               } methods(NUMBER, qw(
-                   new call
+             map { [$_, []] }
+               methods(NUMBER, qw(
                    inc dec not
 
                    factorial
@@ -278,7 +316,11 @@ package Sidef::Optimizer {
 
                    exp int
                    cos sin
-                   log ln log10 log2
+
+                   ln
+                   log
+                   log2
+                   log10
 
                    sin
                    asin
@@ -364,21 +406,19 @@ package Sidef::Optimizer {
 
             # Number.method(Number, Number)
             (
-             map {
-                 { $_, [table(NUMBER), table(NUMBER)] }
-               } methods(NUMBER, qw(
+             map { [$_, [table(NUMBER), table(NUMBER)]] }
+               methods(NUMBER, qw(
                    modpow
                    range
                    )
                )
             ),
-        ],
+        ),
 
-        (BOOL) => [
+        (BOOL) => build_tree(
             (
-             map {
-                 { $_, [] }
-               } methods(BOOL, qw(
+             map { [$_, []] }
+               methods(BOOL, qw(
                    not
                    is_true
                    true
@@ -388,13 +428,12 @@ package Sidef::Optimizer {
                    )
                )
             ),
-        ],
+        ),
 
-        (ARRAY) => [
+        (ARRAY) => build_tree(
             (
-             map {
-                 { $_, [] }
-               } methods(ARRAY, qw(
+             map { [$_, []] }
+               methods(ARRAY, qw(
                    first
                    last
 
@@ -419,9 +458,8 @@ package Sidef::Optimizer {
             ),
 
             (
-             map {
-                 { $_, [table(NUMBER)] }
-               } methods(ARRAY, qw(
+             map { [$_, [table(NUMBER)]] }
+               methods(ARRAY, qw(
                    count
                    index
                    rindex
@@ -430,8 +468,8 @@ package Sidef::Optimizer {
                    contains
                    contains_type
 
-                   divide
-                   multiply
+                   div
+                   mul
 
                    rotate
 
@@ -450,9 +488,8 @@ package Sidef::Optimizer {
             ),
 
             (
-             map {
-                 { $_, [table(ARRAY)] }
-               } methods(ARRAY, qw(
+             map { [$_, [table(ARRAY)]] }
+               methods(ARRAY, qw(
                    and
                    or
                    xor
@@ -468,9 +505,8 @@ package Sidef::Optimizer {
             ),
 
             (
-             map {
-                 { $_, [table(STRING)] }
-               } methods(ARRAY, qw(
+             map { [$_, [table(STRING)]] }
+               methods(ARRAY, qw(
                    pack
                    join
                    index
@@ -484,50 +520,15 @@ package Sidef::Optimizer {
                )
             ),
 
-            (
-             map {
-                 { $_, [table('')] }
-               } methods(ARRAY, qw(reduce_operator))
-            ),
-        ],
+            (map { [$_, [table('')]] } methods(ARRAY, qw(reduce_operator))),
+                             ),
 
-        (MATH) => [
-
-            # Math.method(String)
-            (
-             map {
-                 { $_, [table(STRING)] }
-               } methods(MATH, qw(get_constant))
-            ),
-        ],
-
-        (FILE) => [
-
-            # File.method(String)
-            (
-             map {
-                 { $_, [table(STRING)] }
-               } methods(FILE, qw(get_constant))
-            ),
-        ],
-
-        (SOCKET) => [
-
-            # Socket.method(String)
-            (
-             map {
-                 { $_, [table(STRING)] }
-               } methods(SOCKET, qw(get_constant))
-            ),
-        ],
-
-        (COMPLEX) => [
+        (COMPLEX) => build_tree(
 
             # Complex.method(Complex|Number)
             (
-             map {
-                 { $_, [table(COMPLEX, NUMBER)] }
-               } methods(COMPLEX, qw(
+             map { [$_, [table(COMPLEX, NUMBER)]] }
+               methods(COMPLEX, qw(
                    cmp gt lt ge le eq ne
                    roundf
 
@@ -544,40 +545,17 @@ package Sidef::Optimizer {
                )
             ),
 
-            # Complex.method(Number|Complex, Number|Complex)
-            (
-             map {
-                 { $_, [table(NUMBER, COMPLEX)] }
-               } methods(COMPLEX, qw(
-                   call
-                   new
-                   )
-               )
-            ),
-
-            # Complex.method(Number|Complex, Number|Complex)
-            (
-             map {
-                 { $_, [table(NUMBER, COMPLEX), table(NUMBER, COMPLEX)] }
-               } methods(COMPLEX, qw(
-                   call
-                   new
-                   )
-               )
-            ),
-
             # Complex.method()
             (
-             map {
-                 { $_, [] }
-               } methods(COMPLEX, qw(
-                   new call
+             map { [$_, []] }
+               methods(COMPLEX, qw(
 
                    inc
                    dec
                    abs
 
                    log
+                   log2
                    log10
                    sqrt
 
@@ -606,7 +584,6 @@ package Sidef::Optimizer {
                    asech
                    acoth
 
-                   pi
                    neg
                    not
 
@@ -627,14 +604,170 @@ package Sidef::Optimizer {
                    )
                )
             ),
+        ),
 
-            # Complex.method(String)
+        (NUMBER_DT) => build_tree(
+
+            # Number.method()
             (
-             map {
-                 { $_, [table(STRING)] }
-               } methods(COMPLEX, qw(get_constant))
+             map { [$_, []] }
+               dtypes(NUMBER_DT, qw(
+                   pi
+                   tau
+                   ln2
+                   Y
+                   G
+                   e
+                   phi
+                   nan
+                   inf
+                   ninf
+                   )
+               )
             ),
-        ],
+
+            # Number.method(STRING|NUMBER)
+            (
+             map { [$_, [table(STRING, NUMBER)]] }
+               dtypes(NUMBER_DT, qw(
+                   new
+                   )
+               )
+            ),
+
+            # Number.method(NUMBER, NUMBER)
+            (
+             map { [$_, [table(NUMBER), table(NUMBER)]] }
+               dtypes(NUMBER_DT, qw(
+                   new
+                   )
+               )
+            ),
+
+            # Number.method(STRING, NUMBER)
+            (
+             map { [$_, [table(STRING), table(NUMBER)]] }
+               dtypes(NUMBER_DT, qw(
+                   new
+                   )
+               )
+            ),
+        ),
+
+        (STRING_DT) => build_tree(
+
+            # String.method(STRING|NUMBER)
+            (
+             map { [$_, [table(STRING, NUMBER)]] }
+               dtypes(STRING_DT, qw(
+                   new
+                   )
+               )
+            ),
+        ),
+
+        (REGEX_DT) => build_tree(
+
+            # Regex.method(STRING)
+            (
+             map { [$_, [table(STRING)]] }
+               dtypes(REGEX_DT, qw(
+                   new
+                   )
+               )
+            ),
+        ),
+
+        (RANGENUM_DT) => build_tree(
+
+            # RangeNum.method(NUMBER, NUMBER)
+            (
+             map { [$_, [table(NUMBER), table(NUMBER)]] }
+               dtypes(RANGENUM_DT, qw(
+                   new
+                   )
+               )
+            ),
+
+            # RangeNum.method(NUMBER, NUMBER)
+            (
+             map { [$_, [table(NUMBER), table(NUMBER), table(NUMBER)]] }
+               dtypes(RANGENUM_DT, qw(
+                   new
+                   )
+               )
+            ),
+        ),
+
+        (COMPLEX_DT) => build_tree(
+
+            # Complex.method()
+            (
+             map { [$_, []] }
+               dtypes(COMPLEX_DT, qw(
+                   i
+                   e
+                   pi
+                   phi
+                   new
+                   )
+               )
+            ),
+
+            # Complex.method(STRING|NUMBER)
+            (
+             map { [$_, [table(STRING, NUMBER)]] }
+               dtypes(COMPLEX_DT, qw(
+                   new
+                   )
+               )
+            ),
+
+            # Complex.method(NUMBER|STRING, NUMBER|STRING)
+            (
+             map { [$_, [table(STRING), table(STRING)]] }
+               dtypes(COMPLEX_DT, qw(
+                   new
+                   )
+               )
+            ),
+
+            (
+             map { [$_, [table(STRING), table(NUMBER)]] }
+               dtypes(COMPLEX_DT, qw(
+                   new
+                   )
+               )
+            ),
+
+            (
+             map { [$_, [table(NUMBER), table(STRING)]] }
+               dtypes(COMPLEX_DT, qw(
+                   new
+                   )
+               )
+            ),
+
+            (
+             map { [$_, [table(NUMBER), table(NUMBER)]] }
+               dtypes(COMPLEX_DT, qw(
+                   new
+                   )
+               )
+            ),
+        ),
+
+        (BACKTICK_DT) => build_tree(
+
+            # Backtick.method(STRING)
+            (
+             map { [$_, [table(STRING)]] }
+               dtypes(BACKTICK_DT, qw(
+                   new
+                   )
+               )
+            ),
+        ),
     );
 
     my %addr;
@@ -810,8 +943,8 @@ package Sidef::Optimizer {
                 if (exists $call->{arg}) {
                     foreach my $j (0 .. $#{$call->{arg}}) {
                         my $arg = $call->{arg}[$j];
-                        push @{$obj->{call}[$i]{arg}}, ref $arg eq 'HASH'
-                          && $ref_obj ne 'Sidef::Types::Block::For' ? do { my %arg = $self->optimize($arg); \%arg } : $arg;
+                        push @{$obj->{call}[$i]{arg}},
+                          ref($arg) eq 'HASH' ? do { my %arg = $self->optimize($arg); \%arg } : $arg;
                     }
                 }
 
@@ -830,46 +963,44 @@ package Sidef::Optimizer {
                 my $optimized = 0;
                 if (    defined($ref_obj)
                     and exists($rules{$ref_obj})
-                    and not exists($expr->{ind})
                     and ref($method) eq '') {
 
-                    my $code = $ref_obj->SUPER::can($method);
+                    my $code = ($cache{$ref_obj, $method} //= $ref_obj->SUPER::can($method));
 
                     if (defined $code) {
                         my $obj_call = $obj->{call}[$i];
 
-                        foreach my $rule (@{$rules{$ref_obj}}) {
+                        my $ref = $rules{$ref_obj};
+                        if (exists($ref->{$code}) and (exists($obj_call->{arg}) ? ($#{$obj_call->{arg}} == 0) : 1)) {
+                            $ref = $ref->{$code};
 
-                            if (exists($rule->{$code})
-                                and (exists($obj_call->{arg}) ? ($#{$obj_call->{arg}} == 0) : 1)) {
+                            my @args = (
+                                  exists($obj_call->{arg})
+                                ? ref($obj_call->{arg}[0]) eq 'HASH'
+                                      ? do {
+                                          @{(values(%{$obj_call->{arg}[0]}))[0]};
+                                      }
+                                      : $obj_call->{arg}[0]
+                                : ()
+                            );
 
-                                my @args = (
-                                      exists($obj_call->{arg})
-                                    ? ref($obj_call->{arg}[0]) eq 'HASH'
-                                          ? do {
-                                              @{(values(%{$obj_call->{arg}[0]}))[0]};
-                                          }
-                                          : $obj_call->{arg}[0]
-                                    : ()
-                                );
-
-                                if ($#args == $#{$rule->{$code}}) {
-
-                                    my $ok = 1;
-                                    foreach my $j (0 .. $#args) {
-                                        if (not exists($rule->{$code}[$j]{ref($args[$j])})) {
-                                            $ok = 0;
-                                            last;
-                                        }
+                            if (exists $ref->{$#args}) {
+                                $ref = $ref->{$#args};
+                                my $ok = 1;
+                                foreach my $arg (@args) {
+                                    if (exists $ref->{ref($arg)}) {
+                                        $ref = $ref->{ref($arg)};
                                     }
-
-                                    if ($ok) {
-                                        $obj->{self} = $obj->{self}->$code(@args);
-                                        $ref_obj     = ref($obj->{self});
-                                        $optimized   = 1;
+                                    else {
+                                        $ok = 0;
+                                        last;
                                     }
+                                }
 
-                                    last;
+                                if ($ok) {
+                                    $obj->{self} = $obj->{self}->$code(@args);
+                                    $ref_obj     = ref($obj->{self});
+                                    $optimized   = 1;
                                 }
                             }
                         }
