@@ -15,8 +15,9 @@ package Sidef::Types::Array::Array {
     use Sidef::Types::Number::Number;
 
     sub new {
-        my (undef, @items) = @_;
-        bless \@items, __PACKAGE__;
+        @_ == 2 && ref($_[1]) eq 'ARRAY'
+          ? bless($_[1], __PACKAGE__)
+          : bless [@_[1 .. $#_]], __PACKAGE__;
     }
 
     *call = \&new;
@@ -27,10 +28,10 @@ package Sidef::Types::Array::Array {
         my @array;
         foreach my $item (@{$self}) {
             if (index(ref($item), 'Sidef::') == 0) {
-                push @array, $item->get_value;
+                CORE::push(@array, $item->get_value);
             }
             else {
-                push @array, $item;
+                CORE::push(@array, $item);
             }
         }
 
@@ -48,16 +49,16 @@ package Sidef::Types::Array::Array {
             my $selfc = @{$self};
             my $max   = $argc > $selfc ? $argc - 1 : $selfc - 1;
             foreach my $i (0 .. $max) {
-                push @array, $self->[$i % $selfc]->$operator($arg->[$i % $argc]);
+                CORE::push(@array, $self->[$i % $selfc]->$operator($arg->[$i % $argc]));
             }
         }
         else {
             foreach my $i (0 .. $#{$self}) {
-                push @array, $self->[$i]->$operator;
+                CORE::push(@array, $self->[$i]->$operator);
             }
         }
 
-        $self->new(@array);
+        $self->new(\@array);
     }
 
     sub map_operator {
@@ -67,10 +68,10 @@ package Sidef::Types::Array::Array {
 
         my @array;
         foreach my $i (0 .. $#{$self}) {
-            push @array, $self->[$i]->$operator(@args);
+            CORE::push(@array, $self->[$i]->$operator(@args));
         }
 
-        $self->new(@array);
+        $self->new(\@array);
     }
 
     sub pam_operator {
@@ -80,10 +81,10 @@ package Sidef::Types::Array::Array {
 
         my @array;
         foreach my $i (0 .. $#{$self}) {
-            push @array, $arg->$operator($self->[$i]);
+            CORE::push(@array, $arg->$operator($self->[$i]));
         }
 
-        $self->new(@array);
+        $self->new(\@array);
     }
 
     sub reduce_operator {
@@ -108,19 +109,19 @@ package Sidef::Types::Array::Array {
         if ($operator eq '') {
             foreach my $i (@{$self}) {
                 foreach my $j (@{$arg}) {
-                    push @array, $self->new($i, $j);
+                    CORE::push(@array, $self->new($i, $j));
                 }
             }
         }
         else {
             foreach my $i (@{$self}) {
                 foreach my $j (@{$arg}) {
-                    push @array, $i->$operator($j);
+                    CORE::push(@array, $i->$operator($j));
                 }
             }
         }
 
-        $self->new(@array);
+        $self->new(\@array);
     }
 
     sub zip_operator {
@@ -135,38 +136,19 @@ package Sidef::Types::Array::Array {
         my @array;
         if ($operator eq '') {
             foreach my $i (0 .. $min) {
-                push @array, $self->new($self->[$i], $arg->[$i]);
+                CORE::push(@array, $self->new([$self->[$i], $arg->[$i]]));
             }
         }
         else {
             foreach my $i (0 .. $min) {
-                push @array, $self->[$i]->$operator($arg->[$i]);
+                CORE::push(@array, $self->[$i]->$operator($arg->[$i]));
             }
         }
 
-        $self->new(@array);
+        $self->new(\@array);
     }
 
-    sub _grep {
-        my ($self, $array, $bool) = @_;
-
-        my @new_array;
-        foreach my $item (@{$self}) {
-
-            my $exists = 0;
-            my $value  = $item;
-
-            if ($array->contains($value)) {
-                $exists = 1;
-            }
-
-            push(@new_array, $value) if ($exists - $bool);
-        }
-
-        $self->new(@new_array);
-    }
-
-    sub multiply {
+    sub mul {
         my ($self, $num) = @_;
 
         {
@@ -174,12 +156,10 @@ package Sidef::Types::Array::Array {
             $num = $num->get_value;
         }
 
-        $self->new((@{$self}) x $num);
+        $self->new([(@{$self}) x $num]);
     }
 
-    *mul = \&multiply;
-
-    sub divide {
+    sub div {
         my ($self, $num) = @_;
 
         my @obj = @{$self};
@@ -195,31 +175,97 @@ package Sidef::Types::Array::Array {
         while (@obj) {
             my $j = $pos - $i * int($len);
             $pos -= $j if $j >= 1;
-            push @array, $self->new(splice @obj, 0, $len + $j);
+            CORE::push(@array, $self->new([splice @obj, 0, $len + $j]));
             $pos += $len;
             $i++;
         }
 
-        $self->new(@array);
+        $self->new(\@array);
     }
 
-    *div = \&divide;
-
     sub or {
         my ($self, $array) = @_;
-        my $new_array = $self->new;
-        $self->xor($array)->concat($self->and($array));
+
+        #$self->xor($array)->concat($self->and($array));
+        $self->concat($array)->uniq;
     }
 
     sub xor {
         my ($self, $array) = @_;
-        my $new_array = $self->new;
-        ($self->concat($array))->subtract($self->and($array));
+
+        my @x = sort { $a cmp $b } @{$self};
+        my @y = sort { $a cmp $b } @{$array};
+
+        my $endx = $#x;
+        my $endy = $#y;
+
+        my $i = 0;
+        my $j = 0;
+
+        my ($cmp, @new);
+
+        while (1) {
+
+            $cmp = $x[$i] cmp $y[$j];
+
+            if (ref($cmp) ? $cmp->is_mone : ($cmp == -1)) {
+                push @new, $x[$i];
+                ++$i;
+            }
+            elsif (ref($cmp) ? $cmp->is_one : ($cmp == 1)) {
+                push @new, $y[$j];
+                ++$j;
+            }
+            else {
+                my $k = $i;
+                do { ++$i } while ($i <= $endx and $x[$i] eq $y[$j]);
+                do { ++$j } while ($j <= $endy and $x[$k] eq $y[$j]);
+            }
+
+            if ($i > $endx) {
+                push @new, @y[$j .. $endy];
+                last;
+            }
+            elsif ($j > $endy) {
+                push @new, @x[$i .. $endx];
+                last;
+            }
+        }
+
+        return $self->new(\@new);
     }
 
     sub and {
         my ($self, $array) = @_;
-        $self->_grep($array, 0);
+
+        my @x = sort { $a cmp $b } @{$self};
+        my @y = sort { $a cmp $b } @{$array};
+
+        my $i = 0;
+        my $j = 0;
+
+        my $end1 = @x;
+        my $end2 = @y;
+
+        my ($cmp, @new);
+        while ($i < $end1 and $j < $end2) {
+
+            $cmp = $x[$i] cmp $y[$j];
+
+            if (ref($cmp) ? $cmp->is_mone : ($cmp == -1)) {
+                ++$i;
+            }
+            elsif (ref($cmp) ? $cmp->is_one : ($cmp == 1)) {
+                ++$j;
+            }
+            else {
+                push @new, $x[$i];
+                ++$i;
+                ++$j;
+            }
+        }
+
+        $self->new(\@new);
     }
 
     sub is_empty {
@@ -227,12 +273,41 @@ package Sidef::Types::Array::Array {
         ($#{$self} == -1) ? (Sidef::Types::Bool::Bool::TRUE) : (Sidef::Types::Bool::Bool::FALSE);
     }
 
-    sub subtract {
+    sub diff {
         my ($self, $array) = @_;
-        $self->_grep($array, 1);
-    }
 
-    *sub = \&subtract;
+        my @x = sort { $a cmp $b } @{$self};
+        my @y = sort { $a cmp $b } @{$array};
+
+        my $i = 0;
+        my $j = 0;
+
+        my $end1 = @x;
+        my $end2 = @y;
+
+        my ($cmp, @new);
+        while ($i < $end1 and $j < $end2) {
+
+            $cmp = $x[$i] cmp $y[$j];
+
+            if (ref($cmp) ? $cmp->is_mone : ($cmp == -1)) {
+                push @new, $x[$i];
+                ++$i;
+            }
+            elsif (ref($cmp) ? $cmp->is_one : ($cmp == -1)) {
+                ++$j;
+            }
+            else {
+                1 while (++$i < $end1 and $x[$i] eq $y[$j]);
+            }
+        }
+
+        if ($i < $end1) {
+            push @new, @x[$i .. $#x];
+        }
+
+        $self->new(\@new);
+    }
 
     sub concat {
         my ($self, $arg) = @_;
@@ -339,9 +414,7 @@ package Sidef::Types::Array::Array {
         if (defined($block)) {
 
             if ($k == 0) {
-                if (defined(my $res = $block->_run_code($self->new))) {
-                    return $res;
-                }
+                $block->run($self->new);
                 return $self;
             }
 
@@ -353,11 +426,7 @@ package Sidef::Types::Array::Array {
             my @c = (0 .. $k - 1);
 
             while (1) {
-
-                if (defined(my $res = $block->_run_code($self->new(@{$self}[@c])))) {
-                    return $res;
-                }
-
+                $block->run($self->new(@{$self}[@c]));
                 next if ($c[$k - 1]++ < $n - 1);
                 my $i = $k - 2;
                 $i-- while ($i >= 0 && $c[$i] >= $n - ($k - $i));
@@ -379,7 +448,7 @@ package Sidef::Types::Array::Array {
         my @result;
 
         while (1) {
-            push @result, $self->new(@{$self}[@c]);
+            CORE::push(@result, $self->new(@{$self}[@c]));
             next if ($c[$k - 1]++ < $n - 1);
             my $i = $k - 2;
             $i-- while ($i >= 0 && $c[$i] >= $n - ($k - $i));
@@ -388,7 +457,7 @@ package Sidef::Types::Array::Array {
             while (++$i < $k) { $c[$i] = $c[$i - 1] + 1; }
         }
 
-        $self->new(@result);
+        $self->new(\@result);
     }
 
     *combination = \&combinations;
@@ -446,13 +515,13 @@ package Sidef::Types::Array::Array {
 
             my @tmp_array = ($self->[$i]);
             foreach my $array (@arrays) {
-                push @tmp_array, $array->[$i];
+                CORE::push(@tmp_array, $array->[$i]);
             }
 
-            push @new_array, $self->new(@tmp_array);
+            CORE::push(@new_array, $self->new(\@tmp_array));
         }
 
-        $self->new(@new_array);
+        $self->new(\@new_array);
     }
 
     sub mzip {
@@ -462,19 +531,21 @@ package Sidef::Types::Array::Array {
 
         my @new_array;
         foreach my $i (0 .. $#{$self}) {
-            push @new_array, $self->[$i], $array->[$i % $arr_max];
+            CORE::push(@new_array, $self->[$i], $array->[$i % $arr_max]);
         }
 
-        $self->new(@new_array);
+        $self->new(\@new_array);
     }
 
     sub make {
         my ($self, $size, $type) = @_;
         $self->new(
-            ($type) x do {
-                local $Sidef::Types::Number::Number::GET_PERL_VALUE = 1;
-                $size->get_value;
-              }
+            [
+             ($type) x do {
+                 local $Sidef::Types::Number::Number::GET_PERL_VALUE = 1;
+                 $size->get_value;
+               }
+            ]
         );
     }
 
@@ -617,7 +688,7 @@ package Sidef::Types::Array::Array {
         my @array;
         foreach my $i (0 .. $#{$self}) {
             my $item = $self->[$i];
-            push @array, ref($item) eq ref($self) ? $item->_flatten : $item;
+            CORE::push(@array, ref($item) eq ref($self) ? $item->_flatten : $item);
         }
 
         @array;
@@ -629,10 +700,10 @@ package Sidef::Types::Array::Array {
         my @new_array;
         foreach my $i (0 .. $#{$self}) {
             my $item = $self->[$i];
-            push @new_array, ref($item) eq ref($self) ? ($item->_flatten) : $item;
+            CORE::push(@new_array, ref($item) eq ref($self) ? ($item->_flatten) : $item);
         }
 
-        $self->new(@new_array);
+        $self->new(\@new_array);
     }
 
     *flat = \&flatten;
@@ -667,7 +738,7 @@ package Sidef::Types::Array::Array {
 
     sub items {
         my ($self, @indices) = @_;
-        $self->new(map { exists($self->[$_]) ? $self->[$_] : undef } @indices);
+        $self->new([map { exists($self->[$_]) ? $self->[$_] : undef } @indices]);
     }
 
     sub item {
@@ -722,28 +793,18 @@ package Sidef::Types::Array::Array {
         @{$self}[$from .. $to];
     }
 
-    sub slice {
-        my ($self) = @_;
-        my @items  = _slice(@_);
-        my $array  = $self->new;
-        push @{$array}, @items if @items;
-        $array;
-    }
-
     sub ft {
         my ($self) = @_;
-        $self->new(_slice(@_));
+        $self->new([_slice(@_)]);
     }
 
-    *from_to = \&ft;
+    *slice = \&ft;
 
     sub each {
         my ($self, $code) = @_;
 
         foreach my $item (@{$self}) {
-            if (defined(my $res = $code->_run_code($item))) {
-                return $res;
-            }
+            $code->run($item);
         }
 
         $self;
@@ -762,16 +823,12 @@ package Sidef::Types::Array::Array {
 
         my $end = @{$self};
         for (my $i = $n - 1 ; $i < $end ; $i += $n) {
-            if (defined(my $res = $code->_run_code($self->new(@{$self}[$i - ($n - 1) .. $i])))) {
-                return $res;
-            }
+            $code->run($self->new([@{$self}[$i - ($n - 1) .. $i]]));
         }
 
         my $mod = $end % $n;
         if ($mod != 0) {
-            if (defined(my $res = $code->_run_code($self->new(@{$self}[$end - $mod .. $end - 1])))) {
-                return $res;
-            }
+            $code->run($self->new([@{$self}[$end - $mod .. $end - 1]]));
         }
 
         $self;
@@ -788,15 +845,15 @@ package Sidef::Types::Array::Array {
         my @slices;
         my $end = @{$self};
         for (my $i = $n - 1 ; $i < $end ; $i += $n) {
-            push @slices, $self->new(@{$self}[$i - ($n - 1) .. $i]);
+            CORE::push(@slices, $self->new([@{$self}[$i - ($n - 1) .. $i]]));
         }
 
         my $mod = $end % $n;
         if ($mod != 0) {
-            push @slices, $self->new(@{$self}[$end - $mod .. $end - 1]);
+            CORE::push(@slices, $self->new([@{$self}[$end - $mod .. $end - 1]]));
         }
 
-        $self->new(@slices);
+        $self->new(\@slices);
     }
 
     sub each_cons {
@@ -808,9 +865,7 @@ package Sidef::Types::Array::Array {
         }
 
         foreach my $i ($n - 1 .. $#{$self}) {
-            if (defined(my $res = $code->_run_code($self->new(@{$self}[$i - $n + 1 .. $i])))) {
-                return $res;
-            }
+            $code->run($self->new([@{$self}[$i - $n + 1 .. $i]]));
         }
 
         $self;
@@ -820,9 +875,7 @@ package Sidef::Types::Array::Array {
         my ($self, $code) = @_;
 
         foreach my $i (0 .. $#{$self}) {
-            if (defined(my $res = $code->_run_code(Sidef::Types::Number::Number::_new_uint($i)))) {
-                return $res;
-            }
+            $code->run(Sidef::Types::Number::Number::_new_uint($i));
         }
 
         $self;
@@ -834,9 +887,7 @@ package Sidef::Types::Array::Array {
         my ($self, $code) = @_;
 
         foreach my $i (0 .. $#{$self}) {
-            if (defined(my $res = $code->_run_code(Sidef::Types::Number::Number::_new_uint($i), $self->[$i]))) {
-                return $res;
-            }
+            $code->run(Sidef::Types::Number::Number::_new_uint($i), $self->[$i]);
         }
 
         $self;
@@ -844,7 +895,13 @@ package Sidef::Types::Array::Array {
 
     sub map {
         my ($self, $code) = @_;
-        $self->new(map { $code->run($_) } @{$self});
+
+        my @array;
+        foreach my $item (@{$self}) {
+            CORE::push(@array, $code->run($item));
+        }
+
+        $self->new(\@array);
     }
 
     *collect = \&map;
@@ -854,28 +911,41 @@ package Sidef::Types::Array::Array {
 
         my @arr;
         foreach my $i (0 .. $#{$self}) {
-            push @arr, $code->run(Sidef::Types::Number::Number::_new_uint($i), $self->[$i]);
+            CORE::push(@arr, $code->run(Sidef::Types::Number::Number::_new_uint($i), $self->[$i]));
         }
 
-        $self->new(@arr);
+        $self->new(\@arr);
     }
 
     *collect_kv = \&map_kv;
 
     sub flat_map {
         my ($self, $code) = @_;
-        $self->new(map { @{scalar $code->run($_)} } @{$self});
+
+        my @array;
+        foreach my $item (@{$self}) {
+            CORE::push(@array, @{scalar $code->run($item)});
+        }
+
+        $self->new(\@array);
     }
 
     sub grep {
         my ($self, $obj) = @_;
 
+        my @array;
         if (ref($obj) eq 'Sidef::Types::Regex::Regex') {
-            $self->new(grep { $obj->match($_) } @{$self});
+            foreach my $item (@{$self}) {
+                CORE::push(@array, $item) if $obj->match($item);
+            }
         }
         else {
-            $self->new(grep { scalar $obj->run($_) } @{$self});
+            foreach my $item (@{$self}) {
+                CORE::push(@array, $item) if $obj->run($item);
+            }
         }
+
+        $self->new(\@array);
     }
 
     *filter = \&grep;
@@ -886,10 +956,10 @@ package Sidef::Types::Array::Array {
 
         my @arr;
         foreach my $i (0 .. $#{$self}) {
-            push(@arr, $self->[$i]) if $code->run(Sidef::Types::Number::Number::_new_uint($i), $self->[$i]);
+            CORE::push(@arr, $self->[$i]) if $code->run(Sidef::Types::Number::Number::_new_uint($i), $self->[$i]);
         }
 
-        $self->new(@arr);
+        $self->new(\@arr);
     }
 
     *filter_kv = \&grep_kv;
@@ -902,16 +972,33 @@ package Sidef::Types::Array::Array {
         foreach my $item (@{$self}) {
             my $key = $code->run(my $val = $item);
             my $str_key = "$key";
-            exists($hash->{$str_key}) or do { $hash->{$str_key} = Sidef::Types::Array::Array->new };
-            push @{$hash->{$str_key}}, $val;
+            exists($hash->{$str_key}) or do { $hash->{$str_key} = $self->new };
+            CORE::push(@{$hash->{$str_key}}, $val);
         }
 
         $hash;
     }
 
-    sub freq {
+    sub iter {
         my ($self) = @_;
 
+        my $i = 0;
+        Sidef::Types::Block::Block->new(
+            code => sub {
+                my $result = exists($self->[$i]) ? $self->[$i] : return;
+                ++$i;
+                $result;
+            }
+        );
+    }
+
+    sub freq {
+        my ($self, $code) = @_;
+
+        if (defined($code)) {
+            return $self->freq_by($code);
+        }
+
         my %hash;
         foreach my $item (@{$self}) {
             $hash{$item}++;
@@ -924,7 +1011,20 @@ package Sidef::Types::Array::Array {
         Sidef::Types::Hash::Hash->new(%hash);
     }
 
-    *frequency = \&freq;
+    sub freq_by {
+        my ($self, $code) = @_;
+
+        my %hash;
+        foreach my $item (@{$self}) {
+            $hash{$code->run($item)}++;
+        }
+
+        foreach my $key (keys %hash) {
+            $hash{$key} = Sidef::Types::Number::Number::_new_uint($hash{$key});
+        }
+
+        Sidef::Types::Hash::Hash->new(%hash);
+    }
 
     sub first_by {
         my ($self, $code) = @_;
@@ -979,7 +1079,7 @@ package Sidef::Types::Array::Array {
             }
         }
 
-        $self->new(@values);
+        $self->new(\@values);
     }
 
     sub index {
@@ -1043,10 +1143,10 @@ package Sidef::Types::Array::Array {
 
         my @array;
         for (my $i = 1 ; $i < $end ; $i += 2) {
-            push @array, scalar $obj->run(@{$self}[$i - 1, $i]);
+            CORE::push(@array, scalar $obj->run(@{$self}[$i - 1, $i]));
         }
 
-        $self->new(@array);
+        $self->new(\@array);
     }
 
     sub shuffle {
@@ -1134,7 +1234,7 @@ package Sidef::Types::Array::Array {
                 local $Sidef::Types::Number::Number::GET_PERL_VALUE = 1;
                 $amount = $amount->get_value;
             }
-            return $self->new(map { $self->[CORE::rand(scalar @{$self})] } 1 .. $amount);
+            return $self->new([map { $self->[CORE::rand(scalar @{$self})] } 1 .. $amount]);
         }
         $self->[CORE::rand(scalar @{$self})];
     }
@@ -1160,15 +1260,15 @@ package Sidef::Types::Array::Array {
 
     sub indices {
         my ($self) = @_;
-        Sidef::Types::Array::Array->new(map { Sidef::Types::Number::Number::_new_uint($_) } 0 .. $#{$self});
+        $self->new([map { Sidef::Types::Number::Number::_new_uint($_) } 0 .. $#{$self}]);
     }
 
     *keys = \&indices;
 
     sub pairs {
         my ($self) = @_;
-        __PACKAGE__->new(map { Sidef::Types::Array::Pair->new(Sidef::Types::Number::Number::_new_uint($_), $self->[$_]) }
-                         0 .. $#{$self});
+        $self->new(
+            [map { Sidef::Types::Array::Pair->new(Sidef::Types::Number::Number::_new_uint($_), $self->[$_]) } 0 .. $#{$self}]);
     }
 
     sub insert {
@@ -1193,7 +1293,7 @@ package Sidef::Types::Array::Array {
         my @sorted = do {
             my @arr;
             foreach my $i (0 .. $#{$self}) {
-                push @arr, [$i, $self->[$i]];
+                CORE::push(@arr, [$i, $self->[$i]]);
             }
             CORE::sort { $a->[1] cmp $b->[1] } @arr;
         };
@@ -1206,7 +1306,7 @@ package Sidef::Types::Array::Array {
             ++$i while ($i < $max and $sorted[$i][1] eq $sorted[$i + 1][1]);
         }
 
-        $self->new(grep { defined } @unique);
+        $self->new([grep { defined } @unique]);
     }
 
     *uniq     = \&unique;
@@ -1218,7 +1318,7 @@ package Sidef::Types::Array::Array {
         my @sorted = do {
             my @arr;
             foreach my $i (0 .. $#{$self}) {
-                push @arr, [$i, $self->[$i]];
+                CORE::push(@arr, [$i, $self->[$i]]);
             }
             CORE::sort { $a->[1] cmp $b->[1] } @arr;
         };
@@ -1231,7 +1331,7 @@ package Sidef::Types::Array::Array {
             $unique[$sorted[$i][0]] = $sorted[$i][1];
         }
 
-        $self->new(grep { defined } @unique);
+        $self->new([grep { defined } @unique]);
     }
 
     *last_uniq = \&last_unique;
@@ -1243,7 +1343,7 @@ package Sidef::Types::Array::Array {
             my @arr;
             my $i = -1;
             foreach my $item (@{$self}) {
-                push @arr, [++$i, $item, scalar $block->run($item)];
+                CORE::push(@arr, [++$i, $item, scalar $block->run($item)]);
             }
             CORE::sort { $a->[2] cmp $b->[2] } @arr;
         };
@@ -1256,7 +1356,7 @@ package Sidef::Types::Array::Array {
             ++$i while ($i < $max and $sorted[$i][2] eq $sorted[$i + 1][2]);
         }
 
-        $self->new(grep { defined } @unique);
+        $self->new([grep { defined } @unique]);
     }
 
     *unique_by = \&uniq_by;
@@ -1268,7 +1368,7 @@ package Sidef::Types::Array::Array {
             my @arr;
             my $i = -1;
             foreach my $item (@{$self}) {
-                push @arr, [++$i, $item, scalar $block->run($item)];
+                CORE::push(@arr, [++$i, $item, scalar $block->run($item)]);
             }
             CORE::sort { $a->[2] cmp $b->[2] } @arr;
         };
@@ -1281,7 +1381,7 @@ package Sidef::Types::Array::Array {
             $unique[$sorted[$i][0]] = $sorted[$i][1];
         }
 
-        $self->new(grep { defined } @unique);
+        $self->new([grep { defined } @unique]);
     }
 
     *last_unique_by = \&last_uniq_by;
@@ -1316,10 +1416,10 @@ package Sidef::Types::Array::Array {
                         if ($key eq $tail) {
 
                             if ($callback) {
-                                $block->run($self->new(@{$ref->{$key}}[0 .. $#{$ref->{$key}} - $count]));
+                                $block->run($self->new([@{$ref->{$key}}[0 .. $#{$ref->{$key}} - $count]]));
                             }
                             else {
-                                push @abbrev, $self->new(@{$ref->{$key}}[0 .. $#{$ref->{$key}} - $count]);
+                                CORE::push(@abbrev, $self->new([@{$ref->{$key}}[0 .. $#{$ref->{$key}} - $count]]));
                             }
 
                             last;
@@ -1332,7 +1432,7 @@ package Sidef::Types::Array::Array {
           }
           ->(\%table);
 
-        $self->new(@abbrev);
+        $self->new(\@abbrev);
     }
 
     *abbreviations = \&abbrev;
@@ -1406,14 +1506,16 @@ package Sidef::Types::Array::Array {
 
         if (defined $num) {
             return $self->new(
-                CORE::splice(
-                    @{$self},
-                    0,
-                    do {
-                        local $Sidef::Types::Number::Number::GET_PERL_VALUE = 1;
-                        $num->get_value;
-                      }
-                )
+                [
+                 CORE::splice(
+                     @{$self},
+                     0,
+                     do {
+                         local $Sidef::Types::Number::Number::GET_PERL_VALUE = 1;
+                         $num->get_value;
+                       }
+                 )
+                ]
             );
         }
 
@@ -1434,7 +1536,7 @@ package Sidef::Types::Array::Array {
             }
 
             $num = $num > $#{$self} ? 0 : @{$self} - $num;
-            return $self->new(CORE::splice(@{$self}, $num));
+            return $self->new([CORE::splice(@{$self}, $num)]);
         }
 
         @{$self} || return;
@@ -1474,7 +1576,7 @@ package Sidef::Types::Array::Array {
             $length = defined($length) ? $length->get_value : scalar(@{$self});
         }
 
-        $self->new(CORE::splice(@{$self}, $offset, $length, @objects));
+        $self->new([CORE::splice(@{$self}, $offset, $length, @objects)]);
     }
 
     sub take_right {
@@ -1486,7 +1588,7 @@ package Sidef::Types::Array::Array {
             $amount = $amount->get_value;
             $amount = $end > ($amount - 1) ? $amount - 1 : $end;
         }
-        $self->new(@{$self}[$end - $amount .. $end]);
+        $self->new([@{$self}[$end - $amount .. $end]]);
     }
 
     sub take_left {
@@ -1498,22 +1600,22 @@ package Sidef::Types::Array::Array {
             $amount = $amount->get_value;
             $amount = $end > ($amount - 1) ? $amount - 1 : $end;
         }
-        $self->new(@{$self}[0 .. $amount]);
+        $self->new([@{$self}[0 .. $amount]]);
     }
 
     sub sort {
         my ($self, $code) = @_;
 
         if (defined $code) {
-            return $self->new(CORE::sort { scalar $code->run($a, $b) } @{$self});
+            return $self->new([CORE::sort { scalar $code->run($a, $b) } @{$self}]);
         }
 
-        $self->new(CORE::sort { $a cmp $b } @{$self});
+        $self->new([CORE::sort { $a cmp $b } @{$self}]);
     }
 
     sub sort_by {
         my ($self, $code) = @_;
-        $self->new(map { $_->[0] } sort { $a->[1] cmp $b->[1] } map { [$_, scalar $code->run($_)] } @{$self});
+        $self->new([map { $_->[0] } sort { $a->[1] cmp $b->[1] } map { [$_, scalar $code->run($_)] } @{$self}]);
     }
 
     sub cmp {
@@ -1532,8 +1634,7 @@ package Sidef::Types::Array::Array {
             my $value = $obj1 cmp $obj2;
 
             if (ref($value)) {
-                $value eq Sidef::Types::Number::Number::ZERO
-                  or return $value;
+                $value->is_zero or return $value;
             }
             elsif ($value != 0) {
                 return ($value < 0 ? Sidef::Types::Number::Number::MONE : Sidef::Types::Number::Number::ONE);
@@ -1553,9 +1654,9 @@ package Sidef::Types::Array::Array {
 
         my @array = $self->[0];
         foreach my $i (1 .. $#{$self}) {
-            push @array, $delim_obj, $self->[$i];
+            CORE::push(@array, $delim_obj, $self->[$i]);
         }
-        $self->new(@array);
+        $self->new(\@array);
     }
 
     sub permute {
@@ -1567,25 +1668,21 @@ package Sidef::Types::Array::Array {
         if (defined($code)) {
             my $perm;
             while (1) {
-                $perm = $self->new(@{$self}[@idx]);
+                $perm = $self->new([@{$self}[@idx]]);
 
                 my $p = $#idx;
                 --$p while $idx[$p - 1] > $idx[$p];
 
                 my $q = $p or do {
-                    if (defined(my $res = $code->_run_code($perm))) {
-                        return $res;
-                    }
+                    $code->run($perm);
                     return $self;
                 };
 
-                push @idx, CORE::reverse CORE::splice @idx, $p;
+                CORE::push(@idx, CORE::reverse CORE::splice @idx, $p);
                 ++$q while $idx[$p - 1] > $idx[$q];
                 @idx[$p - 1, $q] = @idx[$q, $p - 1];
 
-                if (defined(my $res = $code->_run_code($perm))) {
-                    return $res;
-                }
+                $code->run($perm);
             }
 
             return;
@@ -1593,11 +1690,11 @@ package Sidef::Types::Array::Array {
 
         my @array;
         while (1) {
-            push @array, $self->new(@{$self}[@idx]);
+            CORE::push(@array, $self->new([@{$self}[@idx]]));
             my $p = $#idx;
             --$p while $idx[$p - 1] > $idx[$p];
-            my $q = $p or (return $self->new(@array));
-            push @idx, CORE::reverse CORE::splice @idx, $p;
+            my $q = $p or (return $self->new(\@array));
+            CORE::push(@idx, CORE::reverse CORE::splice @idx, $p);
             ++$q while $idx[$p - 1] > $idx[$q];
             @idx[$p - 1, $q] = @idx[$q, $p - 1];
         }
@@ -1636,7 +1733,7 @@ package Sidef::Types::Array::Array {
         }
 
         $num %= ($#{$self} + 1);
-        return $self->new(@{$self}) if $num == 0;
+        return $self->new([@{$self}]) if $num == 0;
 
         # Surprisingly, this is slower:
         # $self->new(@{$self}[$num .. $#{$self}], @{$self}[0 .. $num - 1]);
@@ -1644,7 +1741,7 @@ package Sidef::Types::Array::Array {
         # Surprisingly, this is 73% faster:
         my @array = @{$self};
         CORE::unshift(@array, CORE::splice(@array, $num));
-        $self->new(@array);
+        $self->new(\@array);
     }
 
     # Join the array as string
@@ -1674,7 +1771,7 @@ package Sidef::Types::Array::Array {
 
     sub reverse {
         my ($self) = @_;
-        $self->new(CORE::reverse @{$self});
+        $self->new([CORE::reverse @{$self}]);
     }
 
     *reversed = \&reverse;    # alias
@@ -1688,10 +1785,22 @@ package Sidef::Types::Array::Array {
 
     sub copy {
         my ($self) = @_;
-        $self->new(@{$self});
+
+        my @array;
+        my $sub;
+        foreach my $item (@{$self}) {
+            if (ref($item) and defined(eval { $sub = $item->SUPER::can('copy') })) {
+                CORE::push(@array, $sub->($item));
+            }
+            else {
+                CORE::push(@array, $item);
+            }
+        }
+
+        $self->new(\@array);
 
         #state $x = require Storable;
-        #Storable::dclone($self);
+        #Storable::dclone($self);           # this is buggy
     }
 
     sub delete_first {
@@ -1813,7 +1922,7 @@ package Sidef::Types::Array::Array {
         no strict 'refs';
 
         *{__PACKAGE__ . '::' . '&'}   = \&and;
-        *{__PACKAGE__ . '::' . '*'}   = \&multiply;
+        *{__PACKAGE__ . '::' . '*'}   = \&mul;
         *{__PACKAGE__ . '::' . '<<'}  = \&append;
         *{__PACKAGE__ . '::' . '«'}  = \&append;
         *{__PACKAGE__ . '::' . '>>'}  = \&assign_to;
@@ -1821,12 +1930,12 @@ package Sidef::Types::Array::Array {
         *{__PACKAGE__ . '::' . '|'}   = \&or;
         *{__PACKAGE__ . '::' . '^'}   = \&xor;
         *{__PACKAGE__ . '::' . '+'}   = \&concat;
-        *{__PACKAGE__ . '::' . '-'}   = \&subtract;
+        *{__PACKAGE__ . '::' . '-'}   = \&diff;
         *{__PACKAGE__ . '::' . '=='}  = \&eq;
         *{__PACKAGE__ . '::' . '!='}  = \&ne;
         *{__PACKAGE__ . '::' . '<=>'} = \&cmp;
         *{__PACKAGE__ . '::' . ':'}   = \&pair_with;
-        *{__PACKAGE__ . '::' . '/'}   = \&divide;
+        *{__PACKAGE__ . '::' . '/'}   = \&div;
         *{__PACKAGE__ . '::' . '...'} = \&to_list;
 
         *{__PACKAGE__ . '::' . '++'} = sub {
