@@ -43,7 +43,7 @@ package Sidef::Types::Array::MultiArray {
             push @arr, scalar $code->run(map { $_->[$i % @{$_}] } @{$self});
         }
 
-        Sidef::Types::Array::Array->new(@arr);
+        Sidef::Types::Array::Array->new(\@arr);
     }
 
     sub each {
@@ -52,9 +52,7 @@ package Sidef::Types::Array::MultiArray {
         my $max = $self->_max;
 
         foreach my $i (0 .. $max) {
-            if (defined(my $res = $code->_run_code(map { $_->[$i % @{$_}] } @{$self}))) {
-                return $res;
-            }
+            $code->run(map { $_->[$i % @{$_}] } @{$self});
         }
 
         $self;
@@ -72,7 +70,7 @@ package Sidef::Types::Array::MultiArray {
 
     sub to_array {
         my ($self) = @_;
-        Sidef::Types::Array::Array->new(map { Sidef::Types::Array::Array->new(@{$_}) } @{$self});
+        Sidef::Types::Array::Array->new([map { Sidef::Types::Array::Array->new(@{$_}) } @{$self}]);
     }
 
     *to_a = \&to_array;
