@@ -6,6 +6,8 @@ package Sidef::Types::Block::Block {
       Sidef::Convert::Convert
       );
 
+    require List::Util;
+
     use overload
       q{bool} => sub { 1 },
       q{&{}}  => sub { $_[0]->{code} },
@@ -30,12 +32,17 @@ package Sidef::Types::Block::Block {
 
     sub new {
         my (undef, %opt) = @_;
-        bless \%opt, __PACKAGE__;
+
+        bless {
+               name => '__BLOCK__',
+               type => 'block',
+               %opt
+              },
+          __PACKAGE__;
     }
 
     sub run {
-        my ($self, @args) = @_;
-        $self->{code}->(@args);
+        $_[0]{code}->(@_[1 .. $#_]);
     }
 
     *do = \&run;
@@ -87,25 +94,38 @@ package Sidef::Types::Block::Block {
 
             my @pos_args;
             foreach my $var (@vars) {
-                if (exists $var->{type}) {
+                if (exists($var->{type}) or exists($var->{subset})) {
 
                     if (exists $seen{$var->{name}}) {
                         my $value = $seen{$var->{name}};
-                        if (ref($value) eq $var->{type}
-                            or eval { $value->SUPER::isa($var->{type}) }) {
 
-                            if (exists $var->{where_block}) {
-                                $var->{where_block}($value) or next OUTER;
+                        if (exists($var->{type})) {
+                            (ref($value) eq $var->{type} or eval { $value->SUPER::isa($var->{type}) }) || next OUTER;
+
+                            if (exists($var->{where_block})) {
+                                $var->{where_block}($value) || next OUTER;
+                            }
+                            elsif (exists $var->{where_expr}) {
+                                $value eq $var->{where_expr} or next OUTER;
+                            }
+                        }
+
+                        if (exists($var->{subset})) {
+                            $var->{subset}->SUPER::isa(ref($value)) || next OUTER;
+
+                            if (exists($var->{where_block})) {
+                                $var->{where_block}($value) || next OUTER;
                             }
                             elsif (exists $var->{where_expr}) {
                                 $value eq $var->{where_expr} or next OUTER;
                             }
 
-                            push @pos_args, $value;
-                        }
-                        else {
-                            next OUTER;
+                            if (exists $var->{subset_blocks}) {
+                                List::Util::all(sub { $_->($value) }, @{$var->{subset_blocks}}) || next OUTER;
+                            }
                         }
+
+                        push @pos_args, $value;
                     }
                     elsif (exists $var->{has_value}) {
                         push @pos_args, undef;
@@ -115,13 +135,20 @@ package Sidef::Types::Block::Block {
                     }
                 }
                 elsif (exists $seen{$var->{name}}) {
-                    if (exists $var->{where_block}) {
-                        (
-                         exists($var->{slurpy})
-                         ? $var->{where_block}(Sidef::Types::Array::Array->new(@{$seen{$var->{name}}}))
-                         : $var->{where_block}($seen{$var->{name}})
-                        )
-                          or next OUTER;
+                    if (exists($var->{where_block}) or exists($var->{subset_blocks})) {
+
+                        my $value =
+                          exists($var->{slurpy})
+                          ? Sidef::Types::Array::Array->new([@{$seen{$var->{name}}}])
+                          : $seen{$var->{name}};
+
+                        if (exists $var->{where_block}) {
+                            $var->{where_block}($value) || next OUTER;
+                        }
+
+                        if (exists $var->{subset_blocks}) {
+                            List::Util::all(sub { $_->($value) }, @{$var->{subset_blocks}}) || next OUTER;
+                        }
                     }
                     elsif (exists $var->{where_expr}) {
                         $var->{where_expr} eq $seen{$var->{name}} or next OUTER;
@@ -146,14 +173,16 @@ package Sidef::Types::Block::Block {
         my $name = Sidef::normalize_type($self->{name} // '__FUNC__');
 
         die "[ERROR] $self->{type} `$name` does not match $name("
-          . join(', ', map { ref($_) ? Sidef::normalize_type(ref($_)) : 'nil' } @args)
+          . join(', ',
+                 map { ref($_) ? Sidef::normalize_type(ref($_)) : defined($_) ? Sidef::normalize_type($_) : 'nil' } @args)
           . "), invoked as "
           . $name . '('
           . join(
             ', ',
             map {
                     ref($_) && eval { $_->can('dump') } ? $_->dump
-                  : ref($_) ? Sidef::normalize_type(ref($_))
+                  : ref($_)     ? Sidef::normalize_type(ref($_))
+                  : defined($_) ? Sidef::normalize_type($_)
                   : 'nil'
               } @args
           )
@@ -167,8 +196,9 @@ package Sidef::Types::Block::Block {
                     ', ',
                     map {
                             (exists($_->{slurpy}) ? '*' : '')
+                          . (exists($_->{type}) ? (Sidef::normalize_type($_->{type}) . ' ') : '')
                           . $_->{name}
-                          . (exists($_->{type}) ? (": " . Sidef::normalize_type($_->{type})) : '')
+                          . (exists($_->{subset}) ? (' < ' . Sidef::normalize_type($_->{subset})) : '')
                       } @{$_->{vars}}
                     )
               } ($self, (exists($self->{kids}) ? @{$self->{kids}} : ()))
@@ -186,11 +216,6 @@ package Sidef::Types::Block::Block {
 
         my ($self, @objs) = $block->_multiple_dispatch(@args);
 
-        # Unpack 'return'ed values from bare-blocks
-        if (@objs == 1 and ref($objs[0]) eq 'Sidef::Types::Block::Return') {
-            @objs = @{$objs[0]{obj}};
-        }
-
         # Check the return types
         if (exists $self->{returns}) {
 
@@ -258,12 +283,6 @@ package Sidef::Types::Block::Block {
         $num->times($block);
     }
 
-    sub _run_code {
-        my ($self, @args) = @_;
-        my $result = $self->{code}->(@args);
-        ref($result) eq 'Sidef::Types::Block::Return' ? $result : ();
-    }
-
     sub exec {
         my ($self) = @_;
         $self->{code}->();
@@ -274,9 +293,7 @@ package Sidef::Types::Block::Block {
         my ($self, $condition) = @_;
 
         while ($condition->{code}->()) {
-            if (defined(my $res = $self->_run_code)) {
-                return $res;
-            }
+            $self->{code}->();
         }
 
         $self;
@@ -286,9 +303,7 @@ package Sidef::Types::Block::Block {
         my ($self) = @_;
 
         while (1) {
-            if (defined(my $res = $self->_run_code)) {
-                return $res;
-            }
+            $self->{code}->();
         }
 
         $self;
@@ -373,9 +388,7 @@ package Sidef::Types::Block::Block {
         }
         else {
             foreach my $item (@args) {
-                if (defined(my $res = $self->_run_code($item))) {
-                    return $res;
-                }
+                $self->{code}->($item);
             }
             $self;
         }
