@@ -36,7 +36,7 @@ package Sidef::Types::Glob::DirHandle {
         while (defined(my $file = $self->read)) {
             push @files, $file;
         }
-        Sidef::Types::Array::Array->new(@files);
+        Sidef::Types::Array::Array->new(\@files);
     }
 
     *read_dir = \&get_files;
@@ -124,14 +124,23 @@ package Sidef::Types::Glob::DirHandle {
         Sidef::Types::Glob::Stat->lstat($self->{dir_h}, $self);
     }
 
+    sub iter {
+        my ($self) = @_;
+
+        state $x = require Encode;
+        Sidef::Types::Block::Block->new(
+            code => sub {
+                Sidef::Types::String::String->new(Encode::decode_utf8(CORE::readdir($self->{dir_h}) // return));
+            }
+        );
+    }
+
     sub each {
         my ($self, $code) = @_;
 
-        require Encode;
+        state $x = require Encode;
         while (defined(my $file = CORE::readdir($self->{dir_h}))) {
-            if (defined(my $res = $code->_run_code(Sidef::Types::String::String->new(Encode::decode_utf8($file))))) {
-                return $res;
-            }
+            $code->run(Sidef::Types::String::String->new(Encode::decode_utf8($file)));
         }
 
         $self;
