@@ -82,6 +82,15 @@ package Sidef::Types::Glob::FileHandle {
         (CORE::printf {$self->{fh}} @args) ? (Sidef::Types::Bool::Bool::TRUE) : (Sidef::Types::Bool::Bool::FALSE);
     }
 
+    sub iter {
+        my ($self) = @_;
+        Sidef::Types::Block::Block->new(
+            code => sub {
+                Sidef::Types::String::String->new(CORE::readline($self->{fh}) // return);
+            }
+        );
+    }
+
     sub read {
         my ($self, $var_ref, $length, $offset) = @_;
 
@@ -158,7 +167,7 @@ package Sidef::Types::Glob::FileHandle {
         );
     }
 
-    sub readline {
+    sub read_line {
         my ($self, $var_ref) = @_;
 
         if (defined $var_ref) {
@@ -170,18 +179,21 @@ package Sidef::Types::Glob::FileHandle {
         Sidef::Types::String::String->new(CORE::readline($self->{fh}) // return);
     }
 
-    *readln    = \&readline;
-    *read_line = \&readline;
-    *get       = \&readline;
-    *line      = \&readline;
+    *readln   = \&read_line;
+    *readline = \&read_line;
+    *get      = \&read_line;
+    *line     = \&read_line;
 
     sub read_char {
-        my ($self) = @_;
+        my ($self, $var_ref) = @_;
 
-        my $char = getc($self->{fh});
-        defined($char)
-          ? Sidef::Types::String::String->new($char)
-          : ();
+        if (defined $var_ref) {
+            ${$var_ref} =
+              (Sidef::Types::String::String->new(CORE::getc($self->{fh}) // return (Sidef::Types::Bool::Bool::FALSE)));
+            return (Sidef::Types::Bool::Bool::TRUE);
+        }
+
+        Sidef::Types::String::String->new(getc($self->{fh}) // return);
     }
 
     *char = \&read_char;
@@ -189,7 +201,8 @@ package Sidef::Types::Glob::FileHandle {
 
     sub read_lines {
         my ($self) = @_;
-        Sidef::Types::Array::Array->new(map { chomp($_); Sidef::Types::String::String->new($_) } CORE::readline($self->{fh}));
+        Sidef::Types::Array::Array->new(
+                                       [map { chomp($_); Sidef::Types::String::String->new($_) } CORE::readline($self->{fh})]);
     }
 
     *readlines = \&read_lines;
@@ -198,14 +211,14 @@ package Sidef::Types::Glob::FileHandle {
     sub grep {
         my ($self, $obj) = @_;
 
-        my $array = Sidef::Types::Array::Array->new;
+        my @array;
 
         if (ref($obj) eq 'Sidef::Types::Regex::Regex') {
             my $re = $obj->{regex};
             while (defined(my $line = CORE::readline($self->{fh}))) {
                 chomp($line);
                 if ($line =~ $re) {
-                    push @{$array}, Sidef::Types::String::String->new($line);
+                    push @array, Sidef::Types::String::String->new($line);
                 }
             }
         }
@@ -213,42 +226,46 @@ package Sidef::Types::Glob::FileHandle {
             while (defined(my $line = CORE::readline($self->{fh}))) {
                 chomp($line);
                 my $string = Sidef::Types::String::String->new($line);
-                push @{$array}, $string if $obj->run($line);
+                push @array, $string if $obj->run($line);
             }
         }
 
-        $array;
+        Sidef::Types::Array::Array->new(\@array);
     }
 
     sub map {
         my ($self, $block) = @_;
 
-        my $array = Sidef::Types::Array::Array->new;
+        my @array;
         while (defined(my $line = CORE::readline($self->{fh}))) {
             chomp($line);
-            push @{$array}, $block->run(Sidef::Types::String::String->new($line));
+            push @array, $block->run(Sidef::Types::String::String->new($line));
         }
-        $array;
+        Sidef::Types::Array::Array->new(\@array);
     }
 
     sub words {
         my ($self) = @_;
         Sidef::Types::Array::Array->new(
-            map {
-                map    { Sidef::Types::String::String->new($_) }
-                  grep { $_ ne '' }
-                  split(' ', $_)
-              } CORE::readline($self->{fh})
+            [
+             map {
+                 map    { Sidef::Types::String::String->new($_) }
+                   grep { $_ ne '' }
+                   split(' ', $_)
+               } CORE::readline($self->{fh})
+            ]
         );
     }
 
     sub chars {
         my ($self) = @_;
         Sidef::Types::Array::Array->new(
-            map { Sidef::Types::String::String->new($_) } do {
-                local $/;
-                split(//, scalar CORE::readline($self->{fh}));
-              }
+            [
+             map { Sidef::Types::String::String->new($_) } do {
+                 local $/;
+                 split(//, scalar CORE::readline($self->{fh}));
+               }
+            ]
         );
     }
 
@@ -256,9 +273,7 @@ package Sidef::Types::Glob::FileHandle {
         my ($self, $code) = @_;
 
         while (defined(my $line = CORE::readline($self->{fh}))) {
-            if (defined(my $res = $code->_run_code(Sidef::Types::String::String->new($line)))) {
-                return $res;
-            }
+            $code->run(Sidef::Types::String::String->new($line));
         }
 
         $self;
@@ -266,6 +281,16 @@ package Sidef::Types::Glob::FileHandle {
 
     *each_line = \&each;
 
+    sub each_char {
+        my ($self, $code) = @_;
+
+        while (defined(my $char = CORE::getc($self->{fh}))) {
+            $code->run(Sidef::Types::String::String->new($char));
+        }
+
+        $self;
+    }
+
     sub eof {
         my ($self) = @_;
         (eof $self->{fh}) ? (Sidef::Types::Bool::Bool::TRUE) : (Sidef::Types::Bool::Bool::FALSE);
