@@ -26,7 +26,14 @@ package Sidef::Types::Range::RangeNumber {
         my (undef, $from, $to, $step) = @_;
 
         if (defined $to) {
-            Sidef::Types::Number::Number::_valid($from, $to, defined($step) ? $step : ());
+            Sidef::Types::Number::Number::_valid(
+                                                 $from,
+                                                 (
+                                                       ref($to) eq 'Sidef::Types::Number::Inf'
+                                                    || ref($to) eq 'Sidef::Types::Number::Ninf' ? () : $to
+                                                 ),
+                                                 defined($step) ? $step : ()
+                                                );
             $from = ref($from) ? $$from : $from;
             $to   = ref($to)   ? $$to   : $to;
             $step = ref($step) ? $$step : defined($step) ? $step : $ONE;
@@ -176,7 +183,7 @@ package Sidef::Types::Range::RangeNumber {
     *include  = \&contains;
     *includes = \&contains;
 
-    sub range_iter {
+    sub iter {
         my ($self) = @_;
 
         my $step = $self->{step};
@@ -191,28 +198,28 @@ package Sidef::Types::Range::RangeNumber {
         my $is_inf  = Math::GMPq::Rmpq_equal($to, $INF);
         my $is_ninf = Math::GMPq::Rmpq_equal($to, $NINF);
 
-        sub {
-            (
-             $sgn > 0
-             ? ($is_inf ? 1 : $is_ninf ? 0 : Math::GMPq::Rmpq_cmp($i, $to) <= 0)
-             : ($is_inf ? 0 : $is_ninf ? 1 : Math::GMPq::Rmpq_cmp($i, $to) >= 0)
-            )
-              || return;
-            my $tmp = Math::GMPq::Rmpq_init();
-            Math::GMPq::Rmpq_set($tmp, $i);
-            Math::GMPq::Rmpq_add($i, $i, $step);
-            bless \$tmp, 'Sidef::Types::Number::Number';
-        };
+        Sidef::Types::Block::Block->new(
+            code => sub {
+                (
+                 $sgn > 0
+                 ? ($is_inf ? 1 : $is_ninf ? 0 : Math::GMPq::Rmpq_cmp($i, $to) <= 0)
+                 : ($is_inf ? 0 : $is_ninf ? 1 : Math::GMPq::Rmpq_cmp($i, $to) >= 0)
+                )
+                  || return;
+                my $tmp = Math::GMPq::Rmpq_init();
+                Math::GMPq::Rmpq_set($tmp, $i);
+                Math::GMPq::Rmpq_add($i, $i, $step);
+                bless \$tmp, 'Sidef::Types::Number::Number';
+            }
+        );
     }
 
     sub each {
         my ($self, $code) = @_;
 
-        my $iter = $self->range_iter();
+        my $iter = $self->iter();
         while (defined(my $num = $iter->())) {
-            if (defined(my $res = $code->_run_code($num))) {
-                return $res;
-            }
+            $code->run($num);
         }
     }
 
@@ -222,13 +229,13 @@ package Sidef::Types::Range::RangeNumber {
     sub map {
         my ($self, $code) = @_;
 
-        my $values = Sidef::Types::Array::Array->new;
-        my $iter   = $self->range_iter();
+        my @values;
+        my $iter = $self->iter();
         while (defined(my $num = $iter->())) {
-            push @$values, $code->run($num);
+            push @values, $code->run($num);
         }
 
-        $values;
+        Sidef::Types::Array::Array->new(\@values);
     }
 
     *collect = \&map;
@@ -236,13 +243,13 @@ package Sidef::Types::Range::RangeNumber {
     sub grep {
         my ($self, $code) = @_;
 
-        my $values = Sidef::Types::Array::Array->new;
-        my $iter   = $self->range_iter();
+        my @values;
+        my $iter = $self->iter();
         while (defined(my $num = $iter->())) {
-            push(@$values, $num) if $code->run($num);
+            push(@values, $num) if $code->run($num);
         }
 
-        $values;
+        Sidef::Types::Array::Array->new(\@values);
     }
 
     *filter = \&grep;
@@ -251,7 +258,7 @@ package Sidef::Types::Range::RangeNumber {
     sub reduce {
         my ($self, $code) = @_;
 
-        my $iter  = $self->range_iter();
+        my $iter  = $self->iter();
         my $value = $iter->();
 
         while (defined(my $num = $iter->())) {
@@ -264,7 +271,7 @@ package Sidef::Types::Range::RangeNumber {
     sub all {
         my ($self, $code) = @_;
 
-        my $iter = $self->range_iter();
+        my $iter = $self->iter();
         while (defined(my $num = $iter->())) {
             $code->run($num)
               || return Sidef::Types::Bool::Bool::FALSE;
@@ -276,7 +283,7 @@ package Sidef::Types::Range::RangeNumber {
     sub any {
         my ($self, $code) = @_;
 
-        my $iter = $self->range_iter();
+        my $iter = $self->iter();
         while (defined(my $num = $iter->())) {
             $code->run($num)
               && return Sidef::Types::Bool::Bool::TRUE;
@@ -301,13 +308,15 @@ package Sidef::Types::Range::RangeNumber {
 
         my ($name) = (defined($AUTOLOAD) ? ($AUTOLOAD =~ /^.*[^:]::(.*)$/) : '');
 
-        my $array = Sidef::Types::Array::Array->new;
-        my $iter  = $self->range_iter();
+        my @array;
+        my $iter = $self->iter();
         while (defined(my $num = $iter->())) {
-            push @$array, $num;
+            push @array, $num;
         }
 
-        $name eq '' ? $array : $array->$name(@args);
+        $name eq ''
+          ? Sidef::Types::Array::Array->new(\@array)
+          : Sidef::Types::Array::Array->new(\@array)->$name(@args);
     }
 
     {
