@@ -57,11 +57,9 @@ package Sidef::Types::String::String {
          )
           ) < 1
           && return;
-        Sidef::Types::Array::Array->new(map { $self->new($_) } unpack "(a$strlen)*", $$self);
+        Sidef::Types::Array::Array->new([map { bless(\$_, __PACKAGE__) } unpack "(a$strlen)*", $$self]);
     }
 
-    *divide = \&div;
-
     sub lt {
         my ($self, $string) = @_;
         ($$self lt "$string") ? (Sidef::Types::Bool::Bool::TRUE) : (Sidef::Types::Bool::Bool::FALSE);
@@ -82,7 +80,7 @@ package Sidef::Types::String::String {
         ($$self ge "$string") ? (Sidef::Types::Bool::Bool::TRUE) : (Sidef::Types::Bool::Bool::FALSE);
     }
 
-    sub subtract {
+    sub diff {
         my ($self, $obj) = @_;
 
         if (ref($obj) eq 'Sidef::Types::Regex::Regex') {
@@ -148,9 +146,9 @@ package Sidef::Types::String::String {
         my ($s1, $s2) = ($$self, "$string");
 
         if (length($s1) == 1 and length($s2) == 1) {
-            return Sidef::Types::Array::Array->new(map { $self->new(chr($_)) } ord($s1) .. ord($s2));
+            return Sidef::Types::Array::Array->new([map { $self->new(chr($_)) } ord($s1) .. ord($s2)]);
         }
-        Sidef::Types::Array::Array->new(map { $self->new($_) } $s1 .. $s2);
+        Sidef::Types::Array::Array->new([map { $self->new($_) } $s1 .. $s2]);
     }
 
     *arr_to = \&array_to;
@@ -223,7 +221,7 @@ package Sidef::Types::String::String {
         $self->new(~$$self);
     }
 
-    sub times {
+    sub mul {
         my ($self, $num) = @_;
         $self->new(
             $$self x do {
@@ -233,22 +231,18 @@ package Sidef::Types::String::String {
         );
     }
 
-    *multiply = \&times;
-
     sub repeat {
         my ($self, $num) = @_;
         $self->times($num // (Sidef::Types::Number::Number::ONE));
     }
 
-    sub equals {
+    sub eq {
         my ($self, $arg) = @_;
         $$self eq $$arg
           ? (Sidef::Types::Bool::Bool::TRUE)
           : (Sidef::Types::Bool::Bool::FALSE);
     }
 
-    *eq = \&equals;
-
     sub ne {
         my ($self, $arg) = @_;
         $$self ne $$arg
@@ -364,7 +358,7 @@ package Sidef::Types::String::String {
             $string .= CORE::ucfirst(CORE::lc($1)) . $2;
         }
 
-        $self->new($string);
+        bless(\$string, __PACKAGE__);
     }
 
     *wc = \&wordcase;
@@ -467,7 +461,7 @@ package Sidef::Types::String::String {
           : $max;
 
         if (abs($from) > $max) {
-            return __PACKAGE__->new('');
+            return state $x = bless(\(my $str = ''), __PACKAGE__);
         }
 
         if ($to < 0) {
@@ -508,8 +502,7 @@ package Sidef::Types::String::String {
     }
 
     sub clear {
-        my ($self) = @_;
-        $self->new('');
+        state $x = bless(\(my $str = ''), __PACKAGE__);
     }
 
     sub is_empty {
@@ -571,12 +564,12 @@ package Sidef::Types::String::String {
 
     sub sprintf {
         my ($self, @arguments) = @_;
-        __PACKAGE__->new(CORE::sprintf $$self, @arguments);
+        $self->new(CORE::sprintf $$self, @arguments);
     }
 
     sub sprintlnf {
         my ($self, @arguments) = @_;
-        __PACKAGE__->new(CORE::sprintf($$self . "\n", @arguments));
+        $self->new(CORE::sprintf($$self . "\n", @arguments));
     }
 
     sub _string_or_regex {
@@ -595,8 +588,6 @@ package Sidef::Types::String::String {
         ref($str) eq 'Sidef::Types::Block::Block'
           && return $self->esub($regex, $str);
 
-        $str //= __PACKAGE__->new('');
-
         my $search = $self->_string_or_regex($regex);
         my $value  = "$str";
 
@@ -611,8 +602,6 @@ package Sidef::Types::String::String {
         ref($str) eq 'Sidef::Types::Block::Block'
           && return $self->gesub($regex, $str);
 
-        $str //= __PACKAGE__->new('');
-
         my $search = $self->_string_or_regex($regex);
         my $value  = "$str";
         $self->new($$self =~ s{$search}{$value}gr);
@@ -626,40 +615,38 @@ package Sidef::Types::String::String {
     sub esub {
         my ($self, $regex, $code) = @_;
 
-        $code //= __PACKAGE__->new('');
         my $search = $self->_string_or_regex($regex);
 
         if (ref($code) eq 'Sidef::Types::Block::Block') {
-            return __PACKAGE__->new($$self =~ s{$search}{$code->run(_get_captures($$self))}er);
+            return $self->new($$self =~ s{$search}{$code->run(_get_captures($$self))}er);
         }
 
-        __PACKAGE__->new($$self =~ s{$search}{"$code"}eer);
+        $self->new($$self =~ s{$search}{"$code"}eer);
     }
 
     sub gesub {
         my ($self, $regex, $code) = @_;
 
-        $code //= __PACKAGE__->new('');
         my $search = $self->_string_or_regex($regex);
 
         if (ref($code) eq 'Sidef::Types::Block::Block') {
             my $value = $$self;
-            return __PACKAGE__->new($value =~ s{$search}{$code->run(_get_captures($value))}ger);
+            return $self->new($value =~ s{$search}{$code->run(_get_captures($value))}ger);
         }
 
         my $value = "$code";
-        __PACKAGE__->new($$self =~ s{$search}{$value}geer);
+        $self->new($$self =~ s{$search}{$value}geer);
     }
 
     sub glob {
         my ($self) = @_;
         state $x = require Encode;
-        Sidef::Types::Array::Array->new(map { __PACKAGE__->new(Encode::decode_utf8($_)) } CORE::glob($$self));
+        Sidef::Types::Array::Array->new([map { __PACKAGE__->new(Encode::decode_utf8($_)) } CORE::glob($$self)]);
     }
 
     sub quotemeta {
         my ($self) = @_;
-        __PACKAGE__->new(CORE::quotemeta($$self));
+        $self->new(CORE::quotemeta($$self));
     }
 
     *escape = \&quotemeta;
@@ -667,7 +654,7 @@ package Sidef::Types::String::String {
     sub scan {
         my ($self, $regex) = @_;
         my $str = $$self;
-        Sidef::Types::Array::Array->new(map { __PACKAGE__->new($_) } $str =~ /$regex->{regex}/g);
+        Sidef::Types::Array::Array->new([map { bless(\$_, __PACKAGE__) } $str =~ /$regex->{regex}/g]);
     }
 
     sub split {
@@ -682,24 +669,32 @@ package Sidef::Types::String::String {
 
         if (!defined($sep)) {
             return
-              Sidef::Types::Array::Array->new(map { __PACKAGE__->new($_) }
-                                                split(' ', $$self, $size));
+              Sidef::Types::Array::Array->new(
+                                              [map { bless(\$_, __PACKAGE__) }
+                                                 split(' ', $$self, $size)
+                                              ]
+                                             );
         }
 
         if (ref($sep) eq 'Sidef::Types::Number::Number') {
             return Sidef::Types::Array::Array->new(
-                map { __PACKAGE__->new($_) } unpack '(a' . do {
-                    local $Sidef::Types::Number::Number::GET_PERL_VALUE = 1;
-                    $sep->get_value;
-                  }
-                  . ')*',
-                $$self
-                                                  );
+                [
+                 map { bless(\$_, __PACKAGE__) } unpack '(a' . do {
+                     local $Sidef::Types::Number::Number::GET_PERL_VALUE = 1;
+                     $sep->get_value;
+                   }
+                   . ')*',
+                 $$self
+                ]
+            );
         }
 
         $sep = $self->_string_or_regex($sep);
-        Sidef::Types::Array::Array->new(map { __PACKAGE__->new($_) }
-                                          split(/$sep/, $$self, $size));
+        Sidef::Types::Array::Array->new(
+                                        [map { bless(\$_, __PACKAGE__) }
+                                           split(/$sep/, $$self, $size)
+                                        ]
+                                       );
     }
 
     sub sort {
@@ -729,16 +724,14 @@ package Sidef::Types::String::String {
 
     sub words {
         my ($self) = @_;
-        Sidef::Types::Array::Array->new(map { __PACKAGE__->new($_) } CORE::split(' ', $$self));
+        Sidef::Types::Array::Array->new([map { bless(\$_, __PACKAGE__) } CORE::split(' ', $$self)]);
     }
 
     sub each_word {
         my ($self, $code) = @_;
 
         foreach my $word (CORE::split(' ', $$self)) {
-            if (defined(my $res = $code->_run_code(__PACKAGE__->new($word)))) {
-                return $res;
-            }
+            $code->run(bless(\$word, __PACKAGE__));
         }
 
         $self;
@@ -746,7 +739,7 @@ package Sidef::Types::String::String {
 
     sub numbers {
         my ($self) = @_;
-        Sidef::Types::Array::Array->new(map { Sidef::Types::Number::Number->new($_) } CORE::split(' ', $$self));
+        Sidef::Types::Array::Array->new([map { Sidef::Types::Number::Number->new($_) } CORE::split(' ', $$self)]);
     }
 
     *nums = \&numbers;
@@ -755,9 +748,7 @@ package Sidef::Types::String::String {
         my ($self, $code) = @_;
 
         foreach my $num (CORE::split(' ', $$self)) {
-            if (defined(my $res = $code->_run_code(Sidef::Types::Number::Number->new($num)))) {
-                return $res;
-            }
+            $code->run(Sidef::Types::Number::Number->new($num));
         }
 
         $self;
@@ -770,9 +761,11 @@ package Sidef::Types::String::String {
         my $string = $$self;
         state $x = require bytes;
         Sidef::Types::Array::Array->new(
-            map {
-                Sidef::Types::Number::Number::_new_uint(CORE::ord(bytes::substr($string, $_, 1)))
-              } 0 .. bytes::length($string) - 1
+            [
+             map {
+                 Sidef::Types::Number::Number::_new_uint(CORE::ord(bytes::substr($string, $_, 1)))
+               } 0 .. bytes::length($string) - 1
+            ]
         );
     }
 
@@ -783,14 +776,7 @@ package Sidef::Types::String::String {
 
         state $x = require bytes;
         foreach my $i (0 .. bytes::length($string) - 1) {
-            if (
-                defined(
-                        my $res =
-                          $code->_run_code(Sidef::Types::Number::Number::_new_uint(CORE::ord bytes::substr($string, $i, 1)))
-                       )
-              ) {
-                return $res;
-            }
+            $code->run(Sidef::Types::Number::Number::_new_uint(CORE::ord bytes::substr($string, $i, 1)));
         }
 
         $self;
@@ -798,7 +784,7 @@ package Sidef::Types::String::String {
 
     sub chars {
         my ($self) = @_;
-        Sidef::Types::Array::Array->new(map { __PACKAGE__->new($_) } CORE::split(//, CORE::join('', $$self)));
+        Sidef::Types::Array::Array->new([map { bless(\$_, __PACKAGE__) } CORE::split(//, CORE::join('', $$self))]);
     }
 
     *to_a     = \&chars;
@@ -808,9 +794,7 @@ package Sidef::Types::String::String {
         my ($self, $code) = @_;
 
         foreach my $char (CORE::split(//, $$self)) {
-            if (defined(my $res = $code->_run_code(__PACKAGE__->new($char)))) {
-                return $res;
-            }
+            $code->run(bless(\$char, __PACKAGE__));
         }
 
         $self;
@@ -820,7 +804,7 @@ package Sidef::Types::String::String {
 
     sub graphemes {
         my ($self) = @_;
-        Sidef::Types::Array::Array->new(map { __PACKAGE__->new($_) } map { /\X/g } $$self);
+        Sidef::Types::Array::Array->new([map { bless(\$_, __PACKAGE__) } map { /\X/g } $$self]);
     }
 
     *graphs = \&graphemes;
@@ -830,9 +814,7 @@ package Sidef::Types::String::String {
 
         my $str = $$self;
         while ($str =~ /(\X)/g) {
-            if (defined(my $res = $code->_run_code(__PACKAGE__->new($1)))) {
-                return $res;
-            }
+            $code->run(bless(\(my $str = $1), __PACKAGE__));
         }
 
         $self;
@@ -842,16 +824,14 @@ package Sidef::Types::String::String {
 
     sub lines {
         my ($self) = @_;
-        Sidef::Types::Array::Array->new(map { __PACKAGE__->new($_) } CORE::split(/\R/, $$self));
+        Sidef::Types::Array::Array->new([map { bless(\$_, __PACKAGE__) } CORE::split(/\R/, $$self)]);
     }
 
     sub each_line {
         my ($self, $code) = @_;
 
         foreach my $line (CORE::split(/\R/, $$self)) {
-            if (defined(my $res = $code->_run_code(__PACKAGE__->new($line)))) {
-                return $res;
-            }
+            $code->run(bless(\$line, __PACKAGE__));
         }
 
         $self;
@@ -935,13 +915,13 @@ package Sidef::Types::String::String {
 
     sub unpack {
         my ($self, $arg) = @_;
-        my @values = map { __PACKAGE__->new($_) } CORE::unpack($$self, "$arg");
+        my @values = map { bless(\$_, __PACKAGE__) } CORE::unpack($$self, "$arg");
         @values > 1 ? @values : $values[0];
     }
 
     sub pack {
         my ($self, @list) = @_;
-        __PACKAGE__->new(CORE::pack($$self, @list));
+        $self->new(CORE::pack($$self, @list));
     }
 
     sub chars_length {
@@ -1103,7 +1083,7 @@ package Sidef::Types::String::String {
         }
         elsif (ref($arg) eq 'Sidef::Types::Block::Block') {
             foreach my $char (split //, $s) {
-                ++$counter if $arg->run(__PACKAGE__->new($char));
+                ++$counter if $arg->run(bless(\$char, __PACKAGE__));
             }
             return Sidef::Types::Number::Number::_new_uint($counter);
         }
@@ -1471,7 +1451,7 @@ package Sidef::Types::String::String {
         $x || return $self->basic_dump;
 
         local $Data::Dump::TRY_BASE64 = 0;
-        $self->new(Data::Dump::quote($$self) =~ s<(#\{)>{\\$1}gr);
+        __PACKAGE__->new(Data::Dump::quote($$self) =~ s<(#\{)>{\\$1}gr);
     }
 
     *inspect = \&dump;
@@ -1480,11 +1460,11 @@ package Sidef::Types::String::String {
         no strict 'refs';
 
         *{__PACKAGE__ . '::' . '=~'}  = \&match;
-        *{__PACKAGE__ . '::' . '*'}   = \&times;
+        *{__PACKAGE__ . '::' . '*'}   = \&mul;
         *{__PACKAGE__ . '::' . '+'}   = \&append;
         *{__PACKAGE__ . '::' . '++'}  = \&inc;
-        *{__PACKAGE__ . '::' . '-'}   = \&subtract;
-        *{__PACKAGE__ . '::' . '=='}  = \&equals;
+        *{__PACKAGE__ . '::' . '-'}   = \&diff;
+        *{__PACKAGE__ . '::' . '=='}  = \&eq;
         *{__PACKAGE__ . '::' . '!='}  = \&ne;
         *{__PACKAGE__ . '::' . '≠'} = \&ne;
         *{__PACKAGE__ . '::' . '>'}   = \&gt;
